%{
	#include "du1.h"
	#include "du1tok.h"
	#include "duerr.h"
%}

%option noyywrap
%option nostdinit
%option never-interactive
%option 8bit
%option nounput

DIGIT			[0-9]
WHITESPACE		[ \r\n\t\f]
ALPHA           [a-zA-Z]
a				[aA]
b				[bB]
c				[cC]
d				[dD]
e				[eE]
f				[fF]
g				[gG]
h				[hH]
i				[iI]
j				[jJ]
k				[kK]
l				[lL]
m				[mM]
n				[nN]
o				[oO]
p				[pP]
q				[qQ]
r				[rR]
s				[sS]
t				[tT]
u				[uU]
v				[vV]
w				[wW]
x				[xX]
y				[yY]
z				[Zz]

%%

;													return DUTOK_SEMICOLON;
\.													return DUTOK_DOT;
:=													return DUTOK_ASSIGN;
\(													return DUTOK_LPAR;
\)													return DUTOK_RPAR;
:													return DUTOK_COLON;
,													return DUTOK_COMMA;
=													return DUTOK_EQ;
\<=													return DUTOK_OPER_REL;
\>=													return DUTOK_OPER_REL;
\<\>												return DUTOK_OPER_REL;
\<													return DUTOK_OPER_REL;
\>													return DUTOK_OPER_REL;
\+													return DUTOK_OPER_SIGNADD;
\-													return DUTOK_OPER_SIGNADD;
\*													return DUTOK_OPER_MUL;
\/													return DUTOK_OPER_MUL;
{d}{i}{v}											return DUTOK_OPER_MUL;
{m}{o}{d}											return DUTOK_OPER_MUL;
{a}{n}{d}											return DUTOK_OPER_MUL;
{o}{r}												return DUTOK_OR;
{n}{o}{t}											return DUTOK_NOT;

{p}{r}{o}{g}{r}{a}{m}								return DUTOK_PROGRAM;
{l}{a}{b}{e}{l}										return DUTOK_LABEL;
{c}{o}{n}{s}{t}										return DUTOK_CONST;
{t}{y}{p}{e}										return DUTOK_TYPE;
{v}{a}{r}											return DUTOK_VAR;
{b}{e}{g}{i}{n}										return DUTOK_BEGIN;
{e}{n}{d}											return DUTOK_END;
{f}{u}{n}{c}{t}{i}{o}{n}							return DUTOK_FUNCTION;
{p}{r}{o}{c}{e}{d}{u}{r}{e}							return DUTOK_PROCEDURE;
{r}{e}{c}{o}{r}{d}									return DUTOK_RECORD;
{g}{o}{t}{o}										return DUTOK_GOTO;
{i}{f}												return DUTOK_IF;
{t}{h}{e}{n}										return DUTOK_THEN;
{e}{l}{s}{e}										return DUTOK_ELSE;
{w}{h}{i}{l}{e}										return DUTOK_WHILE;
{d}{o}												return DUTOK_DO;
{r}{e}{p}{e}{a}{t}									return DUTOK_REPEAT;
{u}{n}{t}{i}{l}										return DUTOK_UNTIL;
{f}{o}{r}											return DUTOK_FOR;
{t}{o}												return DUTOK_FOR_DIRECTION;
{d}{o}{w}{n}{t}{o}									return DUTOK_FOR_DIRECTION;

\[													return DUTOK_LSBRA;
\]													return DUTOK_RSBRA;
\.\.												return DUTOK_DOTDOT;
{o}{f}												return DUTOK_OF;
{a}{r}{r}{a}{y}										return DUTOK_ARRAY;

{DIGIT}+											return DUTOK_UINT;
({DIGIT}+\.)?{DIGIT}+({e}(\+|-)?{DIGIT}+)?			return DUTOK_REAL;

{ALPHA}({ALPHA}|{DIGIT})*							return DUTOK_IDENTIFIER;

{WHITESPACE}+										/* go out with whitespaces */

.													mlc::error(mlc::DUERR_UNKCHAR, 0, *yytext, *yytext);
