%{
	
	#include "duerr.h"
	#include "du3.h"
	#include "du3tok.h"
	#include "du3g.hpp"
	
	/* portability stuff */
	#if defined _MSC_VER	    /* M$, VS2003 does not recognize long long */
	    typedef unsigned __int64	UINT64;
	    #define C64(n)				n##i64
	    #define STRTOULL			_strtoui64
	#else						/* the rest of the world; hmm, SunCC doesn't have strupr, hopefully no one will use it */
	    typedef unsigned long long	UINT64;
	    #define C64(n)				n##LL
	    #define STRTOULL			strtoull
	#endif
	
	using namespace std;
	using namespace mlc;
	
	unsigned lineCounter  = 1;
	unsigned commentLevel = 0;
	string   globalStr;
	
	unsigned long down31bit (char * digitsInChars, bool * tooBig)
	{
		unsigned long long  result;
		const unsigned long maxInt = 2147483647;
	
		result    = 0;
		(*tooBig) = 0;
		for (; ((0 != (*digitsInChars)) && (('0' <= (*digitsInChars)) && ('9' >= (*digitsInChars)))); digitsInChars++) // ma fungovat jen pro ASCII
		{
			result = 10 * result + (*digitsInChars) - '0'; // ma fungovat pouze pro ASCII
			if (maxInt < result)
				(*tooBig) = 1;
		}
		
		return (unsigned long)(result & maxInt);
	}	
%}

%option noyywrap
%option nostdinit
%option never-interactive
%option 8bit
%option nounput

DIGIT			[0-9]
WHITESPACE		[ \r\n\t\f]
ALPHA           [a-zA-Z]
a				[aA]
b				[bB]
c				[cC]
d				[dD]
e				[eE]
f				[fF]
g				[gG]
h				[hH]
i				[iI]
j				[jJ]
k				[kK]
l				[lL]
m				[mM]
n				[nN]
o				[oO]
p				[pP]
q				[qQ]
r				[rR]
s				[sS]
t				[tT]
u				[uU]
v				[vV]
w				[wW]
x				[xX]
y				[yY]
z				[Zz]
PASTR           ([^\'\n]|\'\')*
PASCMT          ([^\{\}])*

%x				STR
%x				CMT

%%

%{
	string        localStr;
	double        localDbl;
	unsigned long loop1;
	char          eraseState;
	bool          bad;
%}

;													*l = lineCounter; return DUTOK_SEMICOLON;
\.													*l = lineCounter; return DUTOK_DOT;
:=													*l = lineCounter; return DUTOK_ASSIGN;
\(													*l = lineCounter; return DUTOK_LPAR;
\)													*l = lineCounter; return DUTOK_RPAR;
:													*l = lineCounter; return DUTOK_COLON;
,													*l = lineCounter; return DUTOK_COMMA;
=													*l = lineCounter; return DUTOK_EQ;
\<=													*l = lineCounter; lv->dtge_ = DUTOKGE_LE; return DUTOK_OPER_REL;
\>=													*l = lineCounter; lv->dtge_ = DUTOKGE_GE; return DUTOK_OPER_REL;
\<\>												*l = lineCounter; lv->dtge_ = DUTOKGE_NE; return DUTOK_OPER_REL;
\<													*l = lineCounter; lv->dtge_ = DUTOKGE_LT; return DUTOK_OPER_REL;
\>													*l = lineCounter; lv->dtge_ = DUTOKGE_GT; return DUTOK_OPER_REL;
\+													*l = lineCounter; lv->dtge_ = DUTOKGE_PLUS;  return DUTOK_OPER_SIGNADD;
\-													*l = lineCounter; lv->dtge_ = DUTOKGE_MINUS; return DUTOK_OPER_SIGNADD;
\*													*l = lineCounter; lv->dtge_ = DUTOKGE_ASTERISK; return DUTOK_OPER_MUL;
\/													*l = lineCounter; lv->dtge_ = DUTOKGE_SOLIDUS;  return DUTOK_OPER_MUL;
{d}{i}{v}											*l = lineCounter; lv->dtge_ = DUTOKGE_DIV;      return DUTOK_OPER_MUL;
{m}{o}{d}											*l = lineCounter; lv->dtge_ = DUTOKGE_MOD;      return DUTOK_OPER_MUL;
{a}{n}{d}											*l = lineCounter; lv->dtge_ = DUTOKGE_AND;      return DUTOK_OPER_MUL;
{o}{r}												*l = lineCounter; return DUTOK_OR;
{n}{o}{t}											*l = lineCounter; return DUTOK_NOT;

{p}{r}{o}{g}{r}{a}{m}								*l = lineCounter; return DUTOK_PROGRAM;
{l}{a}{b}{e}{l}										*l = lineCounter; return DUTOK_LABEL;
{c}{o}{n}{s}{t}										*l = lineCounter; return DUTOK_CONST;
{t}{y}{p}{e}										*l = lineCounter; return DUTOK_TYPE;
{v}{a}{r}											*l = lineCounter; return DUTOK_VAR;
{b}{e}{g}{i}{n}										*l = lineCounter; return DUTOK_BEGIN;
{e}{n}{d}											*l = lineCounter; return DUTOK_END;
{f}{u}{n}{c}{t}{i}{o}{n}							*l = lineCounter; return DUTOK_FUNCTION;
{p}{r}{o}{c}{e}{d}{u}{r}{e}							*l = lineCounter; return DUTOK_PROCEDURE;
{r}{e}{c}{o}{r}{d}									*l = lineCounter; return DUTOK_RECORD;
{g}{o}{t}{o}										*l = lineCounter; return DUTOK_GOTO;
{i}{f}												*l = lineCounter; return DUTOK_IF;
{t}{h}{e}{n}										*l = lineCounter; return DUTOK_THEN;
{e}{l}{s}{e}										*l = lineCounter; return DUTOK_ELSE;
{w}{h}{i}{l}{e}										*l = lineCounter; return DUTOK_WHILE;
{d}{o}												*l = lineCounter; return DUTOK_DO;
{r}{e}{p}{e}{a}{t}									*l = lineCounter; return DUTOK_REPEAT;
{u}{n}{t}{i}{l}										*l = lineCounter; return DUTOK_UNTIL;
{f}{o}{r}											*l = lineCounter; return DUTOK_FOR;
{t}{o}												*l = lineCounter; lv->dtge_ = DUTOKGE_TO;     return DUTOK_FOR_DIRECTION;
{d}{o}{w}{n}{t}{o}									*l = lineCounter; lv->dtge_ = DUTOKGE_DOWNTO; return DUTOK_FOR_DIRECTION;

\[													*l = lineCounter; return DUTOK_LSBRA;
\]													*l = lineCounter; return DUTOK_RSBRA;
\.\.												*l = lineCounter; return DUTOK_DOTDOT;
{o}{f}												*l = lineCounter; return DUTOK_OF;
{a}{r}{r}{a}{y}										*l = lineCounter; return DUTOK_ARRAY;

{DIGIT}+											{
														lv->int_ci_ = ls_int.add(down31bit(yytext,&bad));
														if (bad)
															error(DUERR_INTOUTRANGE, lineCounter, yytext);
														*l = lineCounter; return DUTOK_UINT;
													}	
({DIGIT}+\.)?{DIGIT}+({e}(\+|-)?{DIGIT}+)?			{
														localDbl     = atof(yytext);
														lv->real_ci_ = ls_real.add(localDbl);
														if ((localDbl != localDbl) || (0 != (localDbl - localDbl))) // nan or infinity
															error(DUERR_REALOUTRANGE, lineCounter, yytext);
														*l = lineCounter; return DUTOK_REAL;
													}	
{DIGIT}+(\.|{e})[^\.]({ALPHA}|{DIGIT})*				{
														localDbl     = atof(yytext);
														lv->real_ci_ = ls_real.add(localDbl);
														error(DUERR_BADREAL, lineCounter, yytext);
														if ((localDbl != localDbl) || (0 != (localDbl - localDbl))) // nan or infinity
															error(DUERR_REALOUTRANGE, lineCounter, yytext);
														*l = lineCounter; return DUTOK_REAL;
													}
{DIGIT}+({ALPHA}|{DIGIT})+							{
														lv->int_ci_ = ls_int.add(down31bit(yytext,&bad));
														error(DUERR_BADINT, lineCounter, yytext);
														if (bad)
															error(DUERR_INTOUTRANGE, lineCounter, yytext);
														*l = lineCounter; return DUTOK_UINT;
													}													

{ALPHA}({ALPHA}|{DIGIT})*							{
														localStr   = yytext;
														transform(localStr.begin(),localStr.end(),localStr.begin(),toupper);
														lv->id_ci_ = ls_id.add(localStr);
														*l = lineCounter; return DUTOK_IDENTIFIER;
													}	

\'													BEGIN(STR);
<STR><<EOF>>										{
														error(DUERR_EOFINSTRCHR, lineCounter);
														lv->str_ci_ = ls_str.add(globalStr);
														globalStr.clear();
														BEGIN(INITIAL);
														*l = lineCounter; return DUTOK_STRING;
													}
<STR>{PASTR}										{
														globalStr.clear();
														globalStr  = yytext;
														eraseState = 0;
														for (loop1 = 0; loop1 < globalStr.length(); loop1++)
														{
															if ('\'' == globalStr[loop1])
																eraseState++;
															else 
																eraseState = 0;
															if (2 == eraseState)
															{
																eraseState = 0;
																globalStr.erase(loop1,1);
																loop1--;
															}		
														}
													}
<STR>\'												{
														lv->str_ci_ = ls_str.add(globalStr);
														globalStr.clear();
														BEGIN(INITIAL);
														*l = lineCounter; return DUTOK_STRING;
													}	
<STR>\n												{
														error(DUERR_EOLINSTRCHR, lineCounter);
														lv->str_ci_ = ls_str.add(globalStr);
														globalStr.clear();
														BEGIN(INITIAL);
														*l = lineCounter; lineCounter++; return DUTOK_STRING;
													}
\{													{
														commentLevel++;
														BEGIN(CMT);
													}
<CMT><<EOF>>										{
														error(DUERR_EOFINCMT, lineCounter);
														BEGIN(INITIAL);
													}													
<CMT>{PASCMT}										{
														for(loop1=0;0!=yytext[loop1];loop1++)
															if ('\n' == yytext[loop1])
																lineCounter++;
													}
<CMT>\{												commentLevel++;
<CMT>\}												{
														commentLevel--;
														if (0 == commentLevel)
														  BEGIN(INITIAL);
													}																										
\}													error(DUERR_UNEXPENDCMT, lineCounter);													

{WHITESPACE}+										{
														for (loop1=0;0!=yytext[loop1];loop1++)
															if ('\n' == yytext[loop1])
																lineCounter++;
													}

.													error(DUERR_UNKCHAR, lineCounter, *yytext, *yytext);	/* -1 should be substituted by current line */
