%{
    #include "du4.h"
    #include "du4lval.hpp"
    #include "duerr.h"
    #include "du4tok.h"
    #include "du4sem.h"

    /* semantic value */
    #define YYSTYPE mlc::MlaskalLval

	/* locations */
	#define YYLTYPE unsigned
	#define YYLLOC_DEFAULT(cur, rhs, n)	do { if(n) (cur)=(rhs)[1]; else (cur)=(rhs)[0]; } while(0)

    /* local stuff */
    using namespace mlc;
    
	/* error stuff */
    #define YYERROR_VERBOSE 1
    static void yyerror(YYLTYPE *line, mlc::MlaskalCtx *ctx, const char *m);
    
%}

%token DUTOK_PROGRAM			    /* program */
%token DUTOK_LABEL			    /* label */
%token DUTOK_CONST			    /* const */
%token DUTOK_TYPE			    /* type */
%token DUTOK_VAR			    /* var */
%token DUTOK_BEGIN			    /* begin */
%token DUTOK_END			    /* end */
%token DUTOK_PROCEDURE			    /* procedure */
%token DUTOK_FUNCTION			    /* function */
%token DUTOK_ARRAY			    /* array */
%token DUTOK_OF				    /* of */
%token DUTOK_GOTO			    /* goto */
%token DUTOK_IF				    /* if */
%token DUTOK_THEN			    /* then */
%token DUTOK_ELSE			    /* else */
%token DUTOK_WHILE			    /* while */
%token DUTOK_DO				    /* do */
%token DUTOK_REPEAT			    /* repeat */
%token DUTOK_UNTIL			    /* until */
%token DUTOK_FOR			    /* for */
%token DUTOK_OR				    /* or */
%token DUTOK_NOT			    /* not */
%token DUTOK_RECORD			    /* record */

/* literals */
%token<id_ci_>   DUTOK_IDENTIFIER			    /* identifier */
%token<int_ci_>  DUTOK_UINT		         	    /* unsigned integer */
%token<real_ci_> DUTOK_REAL			            /* real number */
%token<str_ci_>  DUTOK_STRING			        /* string */

/* delimiters */
%token DUTOK_SEMICOLON			    /* ; */
%token DUTOK_DOT			    /* . */
%token DUTOK_COMMA			    /* , */
%token DUTOK_EQ				    /* = */
%token DUTOK_COLON			    /* : */
%token DUTOK_LPAR			    /* ( */
%token DUTOK_RPAR			    /* ) */
%token DUTOK_DOTDOT			    /* .. */
%token DUTOK_LSBRA			    /* [ */
%token DUTOK_RSBRA			    /* ] */
%token DUTOK_ASSIGN			    /* := */

/* grouped operators and keywords */
%token<dtge_> DUTOK_OPER_REL			    /* <, <=, <>, >=, > */
%token<dtge_> DUTOK_OPER_SIGNADD		    /* +, - */
%token<dtge_> DUTOK_OPER_MUL			    /* *, /, div, mod, and */
%token<dtge_> DUTOK_FOR_DIRECTION		    /* to, downto */

%type<typeVal_>         type structuredTypeDef
%type<varQue_>          variables
%type<formalParamList_> formalParametersList
%type<constVal_>        constant unsignedConstant
%type<fieldList_>       recordItemsList

%start mlaskal
%error-verbose
%pure-parser
%parse-param {mlc::MlaskalCtx *ctx}
%lex-param {mlc::MlaskalCtx *ctx}
%locations

%%



mlaskal:                DUTOK_PROGRAM DUTOK_IDENTIFIER DUTOK_SEMICOLON program_block DUTOK_DOT
		;

program_block:          labelsBlock constantsBlock typesBlock variablesBlock methodsBlock mainBlock
		;
		
labelsBlock:            /* empty */
                       |DUTOK_LABEL labelsList DUTOK_SEMICOLON
        ;                          		
        
labelsList:             DUTOK_UINT                        { ctx->tab->add_label_entry(@1, $1, new_label(ctx)); }
                       |labelsList DUTOK_COMMA DUTOK_UINT { ctx->tab->add_label_entry(@3, $3, new_label(ctx)); }
        ;                               

constantsBlock:         /* empty */
                       |DUTOK_CONST constantsList
        ;                               
        
constantsList:          DUTOK_IDENTIFIER DUTOK_EQ constant DUTOK_SEMICOLON               { myAddConst(ctx, $1, $3, @1); }
                       |constantsList DUTOK_IDENTIFIER DUTOK_EQ constant DUTOK_SEMICOLON { myAddConst(ctx, $2, $4, @2); }
        ;        
        
typesBlock:             /* empty */
                       |DUTOK_TYPE typesList
        ;
        
typesList:              DUTOK_IDENTIFIER DUTOK_EQ type DUTOK_SEMICOLON           { if ($3.defined) ctx->tab->add_type(@1, $1, $3.typePointer); }
                       |typesList DUTOK_IDENTIFIER DUTOK_EQ type DUTOK_SEMICOLON { if ($4.defined) ctx->tab->add_type(@2, $2, $4.typePointer); }
        ;                               
        
variablesBlock:         /* empty */
                       |DUTOK_VAR variablesList
        ;
        
variablesList:          variables DUTOK_COLON type DUTOK_SEMICOLON               { myAddVar(ctx, &($1), $3, @1); }
                       |variablesList variables DUTOK_COLON type DUTOK_SEMICOLON { myAddVar(ctx, &($2), $4, @1); }
        ;  
        
variables:              DUTOK_IDENTIFIER                       { $$.push($1); }
                       |variables DUTOK_COMMA DUTOK_IDENTIFIER { $$ = $1; $$.push($3); }
        ;                                                            
        
methodsBlock:           /* empty */
                       |methodsList
        ;
        
methodsList:            methodHeader DUTOK_SEMICOLON commonBlock DUTOK_SEMICOLON             { ctx->tab->leave(@4) }
                       |methodsList methodHeader DUTOK_SEMICOLON commonBlock DUTOK_SEMICOLON { ctx->tab->leave(@5) }
        ;
        
methodHeader:           functionHeader
                       |procedureHeader
        ;                       

mainBlock:              DUTOK_BEGIN commandsList DUTOK_END
        ;
        
commandsList:           command
                       |commandsList DUTOK_SEMICOLON command
        ;
        
commonBlock:            labelsBlock constantsBlock typesBlock variablesBlock mainBlock
        ;
        
procedureHeader:        DUTOK_PROCEDURE DUTOK_IDENTIFIER                                            { ctx->tab->add_proc(@1, $2, create_parameter_list()); ctx->tab->enter(@2, $2); }
                       |DUTOK_PROCEDURE DUTOK_IDENTIFIER DUTOK_LPAR formalParametersList DUTOK_RPAR { ctx->tab->add_proc(@1, $2, $4                     ); ctx->tab->enter(@2, $2); }
        ;
        
functionHeader:         DUTOK_FUNCTION DUTOK_IDENTIFIER DUTOK_COLON DUTOK_IDENTIFIER /* scalarTypeIdentifier */                                            { myAddFunction(ctx, $2, create_parameter_list(), $4, @2); ctx->tab->enter(@2, $2); }
                       |DUTOK_FUNCTION DUTOK_IDENTIFIER DUTOK_LPAR formalParametersList DUTOK_RPAR DUTOK_COLON DUTOK_IDENTIFIER /* scalarTypeIdentifier */ { myAddFunction(ctx, $2, $4                     , $7, @2); ctx->tab->enter(@2, $2); }
        ;
        
formalParametersList:   variables DUTOK_COLON DUTOK_IDENTIFIER /* typeIdentifier */           { $$ = myCreateParamList(ctx, &($1), $3, false, @3); }
                       |DUTOK_VAR variables DUTOK_COLON DUTOK_IDENTIFIER /* typeIdentifier */ { $$ = myCreateParamList(ctx, &($2), $4, true , @3); }
                       |formalParametersList DUTOK_SEMICOLON variables DUTOK_COLON DUTOK_IDENTIFIER /* typeIdentifier */           { $$ = $1; $$->append_and_kill(myCreateParamList(ctx, &($3), $5, false, @5)); }
                       |formalParametersList DUTOK_SEMICOLON DUTOK_VAR variables DUTOK_COLON DUTOK_IDENTIFIER /* typeIdentifier */ { $$ = $1; $$->append_and_kill(myCreateParamList(ctx, &($4), $6, true , @5)); }
        ;            
        
type:                   /* ordinalType */
                        DUTOK_IDENTIFIER /* typeIdentifier */ /* ordinalTypeIdentifier */ /* structuredTypeIdentifier */ 
                          {
                            mlc::symbol_pointer pType;
                            pType = ctx->tab->find_symbol($1);
                            if (!pType->access_type())
                            {
                              $$.defined     = false;
                            }
                            else
                            {
                              $$.typePointer = pType->access_type()->access_type()->type();
                              $$.defined     = true;
                            }
                          }
                       |structuredTypeDef
        ;
        
/* ordinalType:            DUTOK_IDENTIFIER POZNAMKAordinalTypeIdentifier 
        ; */
        
/* structuredType:         DUTOK_IDENTIFIER POZNAMKAstructuredTypeIdentifier
                       |DUTOK_RECORD recordItemsList DUTOK_SEMICOLON DUTOK_END
                       |DUTOK_RECORD recordItemsList DUTOK_END
                       |DUTOK_RECORD DUTOK_END
        ; */
        
structuredTypeDef:      DUTOK_RECORD recordItemsList DUTOK_SEMICOLON DUTOK_END { $$.typePointer = ctx->tab->create_record_type($2); $$.defined = true; }
                       |DUTOK_RECORD recordItemsList DUTOK_END                 { $$.typePointer = ctx->tab->create_record_type($2); $$.defined = true; }
                       |DUTOK_RECORD DUTOK_END                                 { $$.typePointer = ctx->tab->create_record_type(create_field_list()); $$.defined = true; }
        ;                       
        
recordItemsList:        variables DUTOK_COLON type                                 { $$ = myCreateFieldList(&($1), $3); }
                       |recordItemsList DUTOK_SEMICOLON variables DUTOK_COLON type { $$ = $1; $$->append_and_kill(myCreateFieldList(&($3), $5)); }
        ;
        
command:                goodCommand
                       |evilCommand
        ;                       

goodCommand:            labelForJump goodCmd
                       |goodCmd
        ;
        
evilCommand:            labelForJump evilCmd
                       |evilCmd
        ;                       
        
goodCmd:                assignCmd
                       |procedureCallCmd
                       |gotoCmd
                       |blockCmd
                       |goodConditionCmd
                       |goodWhileCmd
                       |repeatCmd
                       |goodForCmd
                       |nullCmd
        ;
        
evilCmd:                evilConditionCmd
                       |evilWhileCmd
                       |evilForCmd
        ;                               

/*                        assignCmd
                       |procedureCallCmd
                       |gotoCmd
                       |blockCmd
                       |conditionCmd
                       |whileCmd
                       |repeatCmd
                       |forCmd
                       |nullCmd
        ; */
        
labelForJump:           DUTOK_UINT DUTOK_COLON
        ;
        
assignCmd:              variable DUTOK_ASSIGN expression
                      /* |DUTOK_IDENTIFIER POZNAMKAfunctionIdentifier DUTOK_ASSIGN expression - variable muze byt i identifier */
        ;
        
procedureCallCmd:       DUTOK_IDENTIFIER /* procedureIdentifier */
                       |DUTOK_IDENTIFIER /* procedureIdentifier */ DUTOK_LPAR realParameters DUTOK_RPAR
        ;
        
gotoCmd:                DUTOK_GOTO DUTOK_UINT
        ;
        
blockCmd:               DUTOK_BEGIN commandsList DUTOK_END
        ;        
        
goodConditionCmd:       DUTOK_IF booleanExpression DUTOK_THEN goodCommand DUTOK_ELSE goodCommand
        ;
        
evilConditionCmd:       DUTOK_IF booleanExpression DUTOK_THEN command        
                       |DUTOK_IF booleanExpression DUTOK_THEN goodCommand DUTOK_ELSE evilCommand
        ;
        
goodWhileCmd:           DUTOK_WHILE booleanExpression DUTOK_DO goodCommand
        ;
        
evilWhileCmd:           DUTOK_WHILE booleanExpression DUTOK_DO evilCommand
        ;        
        
repeatCmd:              DUTOK_REPEAT commandsList DUTOK_UNTIL booleanExpression
        ;
        
goodForCmd:             DUTOK_FOR DUTOK_IDENTIFIER /* ordinalTypedVariableIdentifier */ DUTOK_ASSIGN ordinalTypedExpression DUTOK_FOR_DIRECTION ordinalTypedExpression DUTOK_DO goodCommand
        ;
        
evilForCmd:             DUTOK_FOR DUTOK_IDENTIFIER /* ordinalTypedVariableIdentifier */ DUTOK_ASSIGN ordinalTypedExpression DUTOK_FOR_DIRECTION ordinalTypedExpression DUTOK_DO evilCommand
        ;        
        
nullCmd:                /* empty */
        ;
        
variable:               DUTOK_IDENTIFIER /* variableIdentifier */
                       |variable /* recordTypedVariable */ DUTOK_DOT DUTOK_IDENTIFIER /* recordItemIdentifier */
        ;
        
IDShortVariable:        /* empty */
                       |IDShortVariable DUTOK_DOT DUTOK_IDENTIFIER /* recordItemIdentifier */
        ;                               
        
realParameters:         expression
                      /* |variable - realParameters nemusi byt promenna nebo vyraz , ackoliv to ve specifikacie je , jenze vyraz je mimo jine i promenna */
                       |realParameters DUTOK_COMMA expression
                      /* |realParameters DUTOK_COMMA variable - opet totez , expression je mimo jine i variable */
        ;
        
expression:             simpleExpression DUTOK_OPER_REL simpleExpression
                       |simpleExpression DUTOK_EQ simpleExpression
                       |simpleExpression
        ;        
        
simpleExpression:       DUTOK_OPER_SIGNADD termsList
                       |termsList
        ;
        
termsList:              termsList DUTOK_OR term
                       |termsList DUTOK_OPER_SIGNADD term
                       |term
        ;
        
term:                   factorsList;
        ;
        
factorsList:            factorsList DUTOK_OPER_MUL factor
                       |factor
        ;
        
factor:                 IDLessUnsignedConstant /* konstanta muze byt jen identifikator , coz zahrnuji ostatni vetve */
                       |DUTOK_IDENTIFIER IDShortVariable /* promenna zacina identifikatorem */
                       |DUTOK_IDENTIFIER /* functionIdentifier */ DUTOK_LPAR realParameters DUTOK_RPAR
                      /* |DUTOK_IDENTIFIER POZNAMKAfunctionIdentifier */ /* IDShoerVariable zahrnuje i moznost empty , takze tim pozere identifier */
                       |DUTOK_LPAR expression DUTOK_RPAR
                       |DUTOK_NOT factor
        ;
        
constant:               unsignedConstant              { $$ = $1; }
                       |DUTOK_OPER_SIGNADD DUTOK_UINT { $$.constType = _INT;  (DUTOKGE_MINUS == $1) ? $$.intVal  = ctx->tab->ls_int().add(-1 * (*$2))  : $$.intVal  = $2; }
                       |DUTOK_OPER_SIGNADD DUTOK_REAL { $$.constType = _REAL; (DUTOKGE_MINUS == $1) ? $$.realVal = ctx->tab->ls_real().add(-1 * (*$2)) : $$.realVal = $2; }
        ;
        
unsignedConstant:       DUTOK_IDENTIFIER /* constantIdentifier */ { $$.constType = _ID;   $$.id      = $1; }
                       |DUTOK_UINT                                { $$.constType = _INT;  $$.intVal  = $1; }
                       |DUTOK_REAL                                { $$.constType = _REAL; $$.realVal = $1; }
                       |DUTOK_STRING                              { $$.constType = _STR;  $$.strVal  = $1; }
        ;
        
IDLessUnsignedConstant: DUTOK_UINT
                       |DUTOK_REAL
                       |DUTOK_STRING
        ;                       
        
/* recordTypedVariable:             DUTOK_IDENTIFIER
        ;
        
variableIdentifier:              DUTOK_IDENTIFIER
        ;
        
ordinalTypedVariableIdentifier:  DUTOK_IDENTIFIER
        ;
        
scalarTypeIdentifier:            DUTOK_IDENTIFIER
        ;
        
ordinalTypeIdentifier:           DUTOK_IDENTIFIER
        ;
        
procedureIdentifier:             DUTOK_IDENTIFIER
        ;

constantIdentifier:              DUTOK_IDENTIFIER
        ;
        
recordItemIdentifier:            DUTOK_IDENTIFIER
        ;
        
functionIdentifier:              DUTOK_IDENTIFIER
        ;
        
structuredTypeIdentifier:        DUTOK_IDENTIFIER
        ;
        
typeIdentifier:                  DUTOK_IDENTIFIER
        ; */
        
booleanExpression:               expression
        ; 
        
ordinalTypedExpression:          expression
        ;             



%%

static void yyerror(unsigned *line, MlaskalCtx *, const char *m)
{
    error(DUERR_SYNTAX, *line, m);
}