/*

DU4SEM.CPP

JY

Mlaskal's semantic interface for DU4

*/

#include "du4sem.h"
#include "du4tok.h"
#include "du4.h"
#include "duerr.h"

namespace mlc {

	void myAddVar(MlaskalCtx * ctx, VariablesQueue * ids, Type definedType, unsigned int line)
	{
		if (definedType.defined)
		{
			while (!(ids->empty()))
			{
				ctx->tab->add_var(line, ids->front(), definedType.typePointer);
				ids->pop();
			}
		}
	};

	parameter_list * myCreateParamList(MlaskalCtx * ctx, VariablesQueue * ids, ls_id_index typeID, bool reference, unsigned int line)
	{
		parameter_list * pParamList;
		symbol_pointer   pTypeName;
		type_pointer     pType;

		pParamList = create_parameter_list();
		pTypeName  = ctx->tab->find_symbol(typeID);

		if ((!pTypeName) || (!pTypeName->access_type()))
		{
			error(DUERR_NOTTYPE, line, *typeID);
			return pParamList;
		}
		else
		{
			pType = pTypeName->access_type()->type();
			while (!(ids->empty()))
			{
				if (reference)
					pParamList->append_parameter_by_reference(ids->front(), pType);
				else
					pParamList->append_parameter_by_value(ids->front(), pType);
				ids->pop();
			}
			return pParamList;
		}
	}

	void             myAddFunction     (MlaskalCtx * ctx, ls_id_index functionID, parameter_list * params, ls_id_index typeID, unsigned int line)
	{
		symbol_pointer pType;

		pType = ctx->tab->find_symbol(typeID);
		if (!((!pType)||(!pType->access_type())))
			ctx->tab->add_fnc(line, functionID, pType->access_type()->type(), params);
		else
		{
			ctx->tab->add_fnc(line, functionID, ctx->tab->logical_undef(), params);
			error(DUERR_NOTTYPE, line, * typeID);
		}
	}

	void             myAddConst        (MlaskalCtx * ctx, ls_id_index constantID, Const constStruct, unsigned int line)
	{
		symbol_pointer pConst;

		if (constStruct.defined)
		{
			if (_ID == constStruct.constType)
			{
				pConst = ctx->tab->find_symbol(constStruct.id);
				if (!((!pConst) || (SKIND_CONST != pConst->kind())))
				{
					switch (pConst->access_const()->type()->cat())
					{
						case TCAT_BOOL: 
							ctx->tab->add_const_bool(line, constantID, pConst->access_const()->access_bool_const()->bool_value());
							break;
						case TCAT_INT:
							ctx->tab->add_const_int(line, constantID, pConst->access_const()->access_int_const()->int_value());
							break;
						case TCAT_REAL:
							ctx->tab->add_const_real(line, constantID, pConst->access_const()->access_real_const()->real_value());
							break;
						case TCAT_STR:
							ctx->tab->add_const_str(line, constantID, pConst->access_const()->access_str_const()->str_value());
							break;
					}
				}
				else
					error(DUERR_NOTCONST, line, *constStruct.id);
			}
			if (_INT == constStruct.constType)
				ctx->tab->add_const_int(line, constantID, constStruct.intVal);
			if (_REAL == constStruct.constType)
				ctx->tab->add_const_real(line, constantID, constStruct.realVal);
			if (_STR == constStruct.constType)
				ctx->tab->add_const_str(line, constantID, constStruct.strVal);
		}
	}

	field_list *     myCreateFieldList (VariablesQueue * ids, Type definedType)
	{
		field_list *   pFieldList;

		pFieldList = create_field_list();

		if (definedType.defined)
		{
			while (!(ids->empty()))
			{
				pFieldList->append_field(ids->front(), definedType.typePointer);
				ids->pop();
			}
			return pFieldList;
		}
		else
			return pFieldList;
	}
}
