/*

DU4SEM.CPP

JY

Mlaskal's semantic interface for DU4

*/

#include "du5sem.h"
#include "du5tok.h"
#include "du5.h"
#include "duerr.h"

namespace mlc {

	void myAddVar(MlaskalCtx * ctx, VariablesQueue * ids, Type definedType, unsigned int line)
	{
		if (definedType.defined)
		{
			while (!(ids->empty()))
			{
				ctx->tab->add_var(line, ids->front(), definedType.typePointer);
				ids->pop();
			}
		}
	};

	parameter_list * myCreateParamList(MlaskalCtx * ctx, VariablesQueue * ids, ls_id_index typeID, bool reference, unsigned int line)
	{
		parameter_list * pParamList;
		symbol_pointer   pTypeName;
		type_pointer     pType;

		pParamList = create_parameter_list();
		pTypeName  = ctx->tab->find_symbol(typeID);

		if ((!pTypeName) || (!pTypeName->access_type()))
		{
			error(DUERR_NOTTYPE, line, *typeID);
			return pParamList;
		}
		else
		{
			pType = pTypeName->access_type()->type();
			while (!(ids->empty()))
			{
				if (reference)
					pParamList->append_parameter_by_reference(ids->front(), pType);
				else
					pParamList->append_parameter_by_value(ids->front(), pType);
				ids->pop();
			}
			return pParamList;
		}
	}

	void             myAddFunction     (MlaskalCtx * ctx, ls_id_index functionID, parameter_list * params, ls_id_index typeID, unsigned int line)
	{
		symbol_pointer pType;

		pType = ctx->tab->find_symbol(typeID);
		if (!((!pType)||(!pType->access_type())))
			ctx->tab->add_fnc(line, functionID, pType->access_type()->type(), params);
		else
		{
			ctx->tab->add_fnc(line, functionID, ctx->tab->logical_undef(), params);
			error(DUERR_NOTTYPE, line, * typeID);
		}
	}

	void             myAddConst        (MlaskalCtx * ctx, ls_id_index constantID, Const constStruct, unsigned int line)
	{
		symbol_pointer pConst;

		if (constStruct.defined)
		{
			if (_ID == constStruct.constType)
			{
				pConst = ctx->tab->find_symbol(constStruct.id);
				if (!((!pConst) || (SKIND_CONST != pConst->kind())))
				{
					switch (pConst->access_const()->type()->cat())
					{
						case TCAT_BOOL: 
							ctx->tab->add_const_bool(line, constantID, pConst->access_const()->access_bool_const()->bool_value());
							break;
						case TCAT_INT:
							ctx->tab->add_const_int(line, constantID, pConst->access_const()->access_int_const()->int_value());
							break;
						case TCAT_REAL:
							ctx->tab->add_const_real(line, constantID, pConst->access_const()->access_real_const()->real_value());
							break;
						case TCAT_STR:
							ctx->tab->add_const_str(line, constantID, pConst->access_const()->access_str_const()->str_value());
							break;
					}
				}
				else
					error(DUERR_NOTCONST, line, *constStruct.id);
			}
			if (_INT == constStruct.constType)
				ctx->tab->add_const_int(line, constantID, constStruct.intVal);
			if (_REAL == constStruct.constType)
				ctx->tab->add_const_real(line, constantID, constStruct.realVal);
			if (_STR == constStruct.constType)
				ctx->tab->add_const_str(line, constantID, constStruct.strVal);
		}
	}

	field_list *     myCreateFieldList (VariablesQueue * ids, Type definedType)
	{
		field_list *   pFieldList;

		pFieldList = create_field_list();

		if (definedType.defined)
		{
			while (!(ids->empty()))
			{
				pFieldList->append_field(ids->front(), definedType.typePointer);
				ids->pop();
			}
			return pFieldList;
		}
		else
			return pFieldList;
	}

	Instruction      myMethodCall      (MlaskalCtx * ctx, ls_id_index methodID, InstructionQueue * params, bool resultRequested, unsigned int line)
	{
		Instruction result;
		symbol_pointer methodSymbolPointer;
		std::stack<mlc::type_category> types;
		parameter_list::const_iterator loop1;

		result.code         = icblock_create();
		result.hasAddress   = false;
		result.typeCat      = TCAT_UNDEF;

		methodSymbolPointer = ctx->tab->find_symbol(methodID);

		if (!methodSymbolPointer)
		{
			while (!((NULL == params) || (params->empty())))
			{
				delete params->front().code;
				params->pop();
			}
			if (resultRequested)
				error(DUERR_NOTFNC, line, * methodID);
			else
				error(DUERR_NOTPROC, line, * methodID);
			return result;
		}
		else
		{
			if ((SKIND_FUNCTION == methodSymbolPointer->kind()) && (resultRequested))
				result.typeCat = methodSymbolPointer->access_function()->type()->cat();
		}

		if (SKIND_FUNCTION == methodSymbolPointer->kind())
		{
			if (TCAT_INT  == result.typeCat)
				result.code->append_instruction(new ai::INITI());
			if (TCAT_REAL == result.typeCat)
				result.code->append_instruction(new ai::INITR());
			if (TCAT_BOOL == result.typeCat)
				result.code->append_instruction(new ai::INITB());
			if (TCAT_STR  == result.typeCat)
				result.code->append_instruction(new ai::INITS());
		}

		if (!((SKIND_FUNCTION  == methodSymbolPointer->kind()) || 
			  (SKIND_PROCEDURE == methodSymbolPointer->kind())))
		{
			if (resultRequested)
				error(DUERR_NOTFNC, line, * methodID);
			else
				error(DUERR_NOTPROC, line, * methodID);
			return result;
		}

		loop1 = methodSymbolPointer->access_subprogram()->parameters()->begin();
		while (methodSymbolPointer->access_subprogram()->parameters()->end() != loop1)
		{
            Instruction   auxIns;
			type_category auxCat;
			
			if ((NULL == params) || (params->empty()))
			{
				error(DUERR_PARNUM, line, * methodID);
				return result;
			}
			
			auxCat = loop1->ltype->cat();
			auxIns = params->front();
			params->pop();

			if (auxIns.typeCat == auxCat)
				types.push(auxCat);
			else
			{
				if ((TCAT_REAL == auxCat) && (TCAT_INT == auxIns.typeCat))
				{
					auxIns.code->append_instruction(new ai::CVRTIR());
					types.push(TCAT_REAL);
				}
				if ((TCAT_REAL == auxIns.typeCat) && (TCAT_INT == auxCat))
				{
					auxIns.code->append_instruction(new ai::CVRTRI());
					types.push(TCAT_INT);
					error(DUERR_CONVERSION, line);
				}
				if (!(((TCAT_REAL == auxCat) && (TCAT_INT == auxIns.typeCat)) ||
					  ((TCAT_REAL == auxIns.typeCat) && (TCAT_INT == auxCat))))
				{
					types.push(auxCat);
				    error(DUERR_CANNOTCONVERT, line);
				}
			}

			icblock_append_delete(result.code, auxIns.code);

			loop1++;
		}

		if ((NULL != params) && (!(params->empty())))
			error(DUERR_PARNUM, line, * methodID);
		while ((NULL != params) && (!(params->empty())))
			params->pop();

		result.code->append_instruction(new ai::CALL(methodSymbolPointer->access_subprogram()->code()));

		while (!(types.empty()))
		{
			if (TCAT_INT  == types.top())
				result.code->append_instruction(new ai::DTORI());
			if (TCAT_BOOL == types.top())
				result.code->append_instruction(new ai::DTORB());
			if (TCAT_REAL == types.top())
				result.code->append_instruction(new ai::DTORR());
			if (TCAT_STR  == types.top())
				result.code->append_instruction(new ai::DTORS());
			types.pop();
		}

		if ((SKIND_FUNCTION == methodSymbolPointer->kind()) && (!(resultRequested)))
		{
			if (TCAT_INT == result.typeCat)
				result.code->append_instruction(new ai::DTORI());
			if (TCAT_BOOL == result.typeCat)
				result.code->append_instruction(new ai::DTORB());
			if (TCAT_REAL == result.typeCat)
				result.code->append_instruction(new ai::DTORR());
			if (TCAT_STR == result.typeCat)
				result.code->append_instruction(new ai::DTORS());
		}

		return result;
	}

	bool             myTotalConversion (type_category target, type_category source, Instruction * result, unsigned int line)
	{
		if ((target) != (source))
        {
			if ((TCAT_REAL == target) && (TCAT_INT == source))
		    {
				result->code->append_instruction(new ai::CVRTIR());
			    return true;
		    }
			if ((TCAT_REAL == source) && (TCAT_INT == target))
            {
                result->code->append_instruction(new ai::CVRTRI());
                error(DUERR_CONVERSION, line);
			    return false;
            }
            error(DUERR_CANNOTCONVERT, line);
		    return false;
        }
		else
			return true;
	}

	icblock_pointer myRelOperResult   (int relOper, Instruction * left, Instruction * right, unsigned int line)
	{
		icblock_pointer result;
		type_category   unitedType;
		
		if (left->typeCat == right->typeCat)
			unitedType = left->typeCat;
		else
			unitedType = TCAT_UNDEF;

		if (TCAT_UNDEF == unitedType)
		{
			if (((TCAT_INT ==  left->typeCat) || (TCAT_REAL ==  left->typeCat)) &&
				((TCAT_INT == right->typeCat) || (TCAT_REAL == right->typeCat)))
			{
				unitedType = TCAT_REAL;
				if (TCAT_INT == left->typeCat)
					myTotalConversion(right->typeCat, left->typeCat, left, line);
				if (TCAT_INT == right->typeCat)
					myTotalConversion(left->typeCat, right->typeCat, right, line);
			}
		}

		result = left->code;
		icblock_append_delete(result, right->code);



		if ((-1) == relOper)
		{
			if (TCAT_INT  == unitedType)
				result->append_instruction(new ai::EQI());
			if (TCAT_BOOL == unitedType)
				result->append_instruction(new ai::EQB());
			if (TCAT_REAL == unitedType)
				result->append_instruction(new ai::EQR());
			if (TCAT_STR  == unitedType)
				result->append_instruction(new ai::EQS());
		}

		if (DUTOKGE_NE == relOper)
		{
			if (TCAT_INT  == unitedType)
				result->append_instruction(new ai::NEI());
			if (TCAT_BOOL == unitedType)
				result->append_instruction(new ai::NEB());
			if (TCAT_REAL == unitedType)
				result->append_instruction(new ai::NER());
			if (TCAT_STR  == unitedType)
				result->append_instruction(new ai::NES());
		}

		if (DUTOKGE_LT == relOper)
		{
			if (TCAT_INT  == unitedType)
				result->append_instruction(new ai::LTI());
			if (TCAT_BOOL == unitedType)
				result->append_instruction(new ai::LTB());
			if (TCAT_REAL == unitedType)
				result->append_instruction(new ai::LTR());
			if (TCAT_STR  == unitedType)
				result->append_instruction(new ai::LTS());
		}

		if (DUTOKGE_LE == relOper)
		{
			if (TCAT_INT  == unitedType)
				result->append_instruction(new ai::LEI());
			if (TCAT_BOOL == unitedType)
				result->append_instruction(new ai::LEB());
			if (TCAT_REAL == unitedType)
				result->append_instruction(new ai::LER());
			if (TCAT_STR  == unitedType)
				result->append_instruction(new ai::LES());
		}

		if (DUTOKGE_GE == relOper)
		{
			if (TCAT_INT  == unitedType)
				result->append_instruction(new ai::GEI());
			if (TCAT_BOOL == unitedType)
				result->append_instruction(new ai::GEB());
			if (TCAT_REAL == unitedType)
				result->append_instruction(new ai::GER());
			if (TCAT_STR  == unitedType)
				result->append_instruction(new ai::GES());
		}

		if (DUTOKGE_GT == relOper)
		{
			if (TCAT_INT  == unitedType)
				result->append_instruction(new ai::GTI());
			if (TCAT_BOOL == unitedType)
				result->append_instruction(new ai::GTB());
			if (TCAT_REAL == unitedType)
				result->append_instruction(new ai::GTR());
			if (TCAT_STR  == unitedType)
				result->append_instruction(new ai::GTS());
		}

		if (TCAT_UNDEF == unitedType)
			error(DUERR_CANNOTCONVERT, line);

		return result;
	}

	Instruction      myTermsListBuild  (int termOper, Instruction * terms, Instruction * term, unsigned int line)
	{
		Instruction result;

		result.hasAddress = false;

		if (terms->typeCat == term->typeCat)
			result.typeCat = terms->typeCat;
		else
			result.typeCat = TCAT_UNDEF;

		if (TCAT_UNDEF == result.typeCat)
		{
			if (((TCAT_INT == terms->typeCat) || (TCAT_REAL == terms->typeCat)) &&
				((TCAT_INT ==  term->typeCat) || (TCAT_REAL ==  term->typeCat)))
			{
				result.typeCat = TCAT_REAL;
				if (TCAT_INT == terms->typeCat)
					myTotalConversion(term->typeCat, terms->typeCat, terms, line);
				if (TCAT_INT == term->typeCat)
					myTotalConversion(terms->typeCat, term->typeCat, term, line);
			}
		}

		result.code = terms->code;
		icblock_append_delete(result.code, term->code);

		if (DUTOKGE_PLUS == termOper)
		{
			if (TCAT_INT == result.typeCat)
				result.code->append_instruction(new ai::ADDI());
			if (TCAT_BOOL == result.typeCat)
				error(DUERR_CANNOTCONVERT, line);
			if (TCAT_REAL == result.typeCat)
				result.code->append_instruction(new ai::ADDR());
			if (TCAT_STR == result.typeCat)
				result.code->append_instruction(new ai::ADDS());
		}

		if (DUTOKGE_MINUS == termOper)
		{
			if (TCAT_INT == result.typeCat)
				result.code->append_instruction(new ai::SUBI());
			if (TCAT_BOOL == result.typeCat)
				error(DUERR_CANNOTCONVERT, line);
			if (TCAT_REAL == result.typeCat)
				result.code->append_instruction(new ai::SUBR());
			if (TCAT_STR == result.typeCat)
				error(DUERR_CANNOTCONVERT, line);
		}

		if ((-1) == termOper)
		{
			if (TCAT_INT == result.typeCat)
				error(DUERR_CANNOTCONVERT, line);
			if (TCAT_BOOL == result.typeCat)
				result.code->append_instruction(new ai::OR());
			if (TCAT_REAL == result.typeCat)
				error(DUERR_CANNOTCONVERT, line);
			if (TCAT_STR == result.typeCat)
				error(DUERR_CANNOTCONVERT, line);
		}

		return result;
	}

	Instruction      myFactorsListBuild(int factorOper, Instruction * factors, Instruction * factor, unsigned int line)
	{
		Instruction result;

		result.hasAddress = false;

		if (DUTOKGE_SOLIDUS == factorOper)
		{
			if (TCAT_INT == factors->typeCat)
				factors->code->append_instruction(new ai::CVRTIR());
			if (TCAT_INT == factor->typeCat)
				factor->code->append_instruction(new ai::CVRTIR());
		}

		if (factors->typeCat == factor->typeCat)
			result.typeCat = factors->typeCat;
		else
			result.typeCat = TCAT_UNDEF;

		if (TCAT_UNDEF == result.typeCat)
		{
			if (((TCAT_INT == factors->typeCat) || (TCAT_REAL == factors->typeCat)) &&
				((TCAT_INT ==  factor->typeCat) || (TCAT_REAL ==  factor->typeCat)))
			{
				result.typeCat = TCAT_REAL;
				if (TCAT_INT == factors->typeCat)
					myTotalConversion(factor->typeCat, factors->typeCat, factors, line);
				if (TCAT_INT == factor->typeCat)
					myTotalConversion(factors->typeCat, factor->typeCat, factor, line);
			}
		}

		result.code = factors->code;
		icblock_append_delete(result.code, factor->code);

		if (DUTOKGE_AND == factorOper)
		{
			if (TCAT_INT == result.typeCat)
				error(DUERR_CANNOTCONVERT, line);
			if (TCAT_BOOL == result.typeCat)
				result.code->append_instruction(new ai::AND());
			if (TCAT_REAL == result.typeCat)
				error(DUERR_CANNOTCONVERT, line);
			if (TCAT_STR == result.typeCat)
				error(DUERR_CANNOTCONVERT, line);
		}

		if (DUTOKGE_ASTERISK == factorOper)
		{
			if (TCAT_INT == result.typeCat)
				result.code->append_instruction(new ai::MULI());
			if (TCAT_BOOL == result.typeCat)
				error(DUERR_CANNOTCONVERT, line);
			if (TCAT_REAL == result.typeCat)
				result.code->append_instruction(new ai::MULR());
			if (TCAT_STR == result.typeCat)
				error(DUERR_CANNOTCONVERT, line);
		}

		if (DUTOKGE_SOLIDUS == factorOper)
		{
			if (TCAT_INT == result.typeCat)
				error(DUERR_CANNOTCONVERT, line); // nemuze nastat - na zacatku by to bylo prevedeno na real
			if (TCAT_BOOL == result.typeCat)
				error(DUERR_CANNOTCONVERT, line);
			if (TCAT_REAL == result.typeCat)
				result.code->append_instruction(new ai::DIVR());
			if (TCAT_STR == result.typeCat)
				error(DUERR_CANNOTCONVERT, line);
		}

		if (DUTOKGE_DIV == factorOper)
		{
			if (TCAT_INT == result.typeCat)
				result.code->append_instruction(new ai::DIVI());
			if (TCAT_BOOL == result.typeCat)
				error(DUERR_CANNOTCONVERT, line);
			if (TCAT_REAL == result.typeCat)
				error(DUERR_CANNOTCONVERT, line);
			if (TCAT_STR == result.typeCat)
				error(DUERR_CANNOTCONVERT, line);
		}

		if (DUTOKGE_MOD == factorOper)
		{
			if (TCAT_INT == result.typeCat)
				result.code->append_instruction(new ai::MODI());
			if (TCAT_BOOL == result.typeCat)
				error(DUERR_CANNOTCONVERT, line);
			if (TCAT_REAL == result.typeCat)
				error(DUERR_CANNOTCONVERT, line);
			if (TCAT_STR == result.typeCat)
				error(DUERR_CANNOTCONVERT, line);
		}

		return result;
	}

	Instruction      myIdentifierProcess (MlaskalCtx * ctx, ls_id_index commonID, bool stor, unsigned int line)
	{
		Instruction result;
		symbol_pointer symbolPointer;

		symbolPointer = ctx->tab->find_symbol(commonID);

		if (!(!symbolPointer))
		{
			if (SKIND_CONST == symbolPointer->kind())
			{
				result.code       = icblock_create();
				result.typeCat    = symbolPointer->access_const()->type()->cat();
				if (stor)
				{
					error(DUERR_NOTLVALUE, line);
					result.hasAddress = true;
					return result;
				}
				else
					result.hasAddress = false;
				if (TCAT_INT == result.typeCat)
					result.code->append_instruction(new ai::LDLITI(symbolPointer->access_const()->access_int_const()->int_value()));
				if (TCAT_BOOL == result.typeCat)
					result.code->append_instruction(new ai::LDLITB(symbolPointer->access_const()->access_bool_const()->bool_value()));
				if (TCAT_REAL == result.typeCat)
					result.code->append_instruction(new ai::LDLITR(symbolPointer->access_const()->access_real_const()->real_value()));
				if (TCAT_STR == result.typeCat)
					result.code->append_instruction(new ai::LDLITS(symbolPointer->access_const()->access_str_const()->str_value()));
				return result;
			}
			if (SKIND_FUNCTION == symbolPointer->kind())
			{
				if (stor)
				{
					if (ctx->tab->my_function_name() == commonID)
					{
						result.code       = icblock_create();
						result.typeCat    = symbolPointer->access_function()->type()->cat();
						result.hasAddress = true;
						if (TCAT_INT == result.typeCat)
							result.code->append_instruction(new ai::LSTI(ctx->tab->my_return_address()));
						if (TCAT_BOOL == result.typeCat)
							result.code->append_instruction(new ai::LSTB(ctx->tab->my_return_address()));
						if (TCAT_REAL == result.typeCat)
							result.code->append_instruction(new ai::LSTR(ctx->tab->my_return_address()));
						if (TCAT_STR == result.typeCat)
							result.code->append_instruction(new ai::LSTS(ctx->tab->my_return_address()));
						return result;
					}
					else
					{
						error(DUERR_NOTLVALUE, line);
						result.code       = icblock_create();
						result.typeCat    = TCAT_UNDEF;
						result.hasAddress = false;
						return result;
					}
				}
				else
				{
					result = myMethodCall(ctx, commonID, NULL, true, line);
					return result;
				}
			}
			if (SKIND_GLOBAL_VARIABLE == symbolPointer->kind())
			{
				result.code       = icblock_create();
				result.typeCat    = symbolPointer->access_variable()->type()->cat();
				if (stor)
					result.hasAddress = true;
				else
					result.hasAddress = false;
				if ((TCAT_INT == result.typeCat) && (stor))
					result.code->append_instruction(new ai::GSTI(symbolPointer->access_variable()->address()));
				if ((TCAT_INT == result.typeCat) && (!stor))
					result.code->append_instruction(new ai::GLDI(symbolPointer->access_variable()->address()));
				if ((TCAT_BOOL == result.typeCat) && (stor))
					result.code->append_instruction(new ai::GSTB(symbolPointer->access_variable()->address()));
				if ((TCAT_BOOL == result.typeCat) && (!stor))
					result.code->append_instruction(new ai::GLDB(symbolPointer->access_variable()->address()));
				if ((TCAT_REAL == result.typeCat) && (stor))
					result.code->append_instruction(new ai::GSTR(symbolPointer->access_variable()->address()));
				if ((TCAT_REAL == result.typeCat) && (!stor))
					result.code->append_instruction(new ai::GLDR(symbolPointer->access_variable()->address()));
				if ((TCAT_STR == result.typeCat) && (stor))
					result.code->append_instruction(new ai::GSTS(symbolPointer->access_variable()->address()));
				if ((TCAT_STR == result.typeCat) && (!stor))
					result.code->append_instruction(new ai::GLDS(symbolPointer->access_variable()->address()));
				return result;
			}
			if (SKIND_LOCAL_VARIABLE == symbolPointer->kind())
			{
				result.code       = icblock_create();
				result.typeCat    = symbolPointer->access_variable()->type()->cat();
				if (stor)
					result.hasAddress = true;
				else
					result.hasAddress = false;
				if ((TCAT_INT == result.typeCat) && (stor))
					result.code->append_instruction(new ai::LSTI(symbolPointer->access_variable()->address()));
				if ((TCAT_INT == result.typeCat) && (!stor))
					result.code->append_instruction(new ai::LLDI(symbolPointer->access_variable()->address()));
				if ((TCAT_BOOL == result.typeCat) && (stor))
					result.code->append_instruction(new ai::LSTB(symbolPointer->access_variable()->address()));
				if ((TCAT_BOOL == result.typeCat) && (!stor))
					result.code->append_instruction(new ai::LLDB(symbolPointer->access_variable()->address()));
				if ((TCAT_REAL == result.typeCat) && (stor))
					result.code->append_instruction(new ai::LSTR(symbolPointer->access_variable()->address()));
				if ((TCAT_REAL == result.typeCat) && (!stor))
					result.code->append_instruction(new ai::LLDR(symbolPointer->access_variable()->address()));
				if ((TCAT_STR == result.typeCat) && (stor))
					result.code->append_instruction(new ai::LSTS(symbolPointer->access_variable()->address()));
				if ((TCAT_STR == result.typeCat) && (!stor))
					result.code->append_instruction(new ai::LLDS(symbolPointer->access_variable()->address()));
				return result;
			}
			if (SKIND_PROCEDURE == symbolPointer->kind())
			{
				error(DUERR_NOTVAR, line);
				result.code       = icblock_create();
				result.typeCat    = TCAT_UNDEF;
				result.hasAddress = false;
				return result;
			}
			if (SKIND_TYPE == symbolPointer->kind())
			{
				error(DUERR_NOTVAR, line);
				result.code       = icblock_create();
				result.typeCat    = TCAT_UNDEF;
				result.hasAddress = false;
				return result;
			}
			if (SKIND_UNDEF == symbolPointer->kind())
			{
				error(DUERR_NOTVAR, line);
				result.code       = icblock_create();
				result.typeCat    = TCAT_UNDEF;
				result.hasAddress = false;
				return result;
			}
			result.code       = icblock_create();
			result.typeCat    = TCAT_UNDEF;
			result.hasAddress = false;
			return result;
		}
		else
		{
			error(DUERR_NOTVAR, line, * commonID);
			result.code       = icblock_create();
			result.typeCat    = TCAT_UNDEF;
			result.hasAddress = false;
			return result;
		}
	}

void             myDebug             ()
	{
		return;
	}
}