%{
    #include "du5.h"
    #include "du5lval.hpp"
    #include "duerr.h"
    #include "du5tok.h"
    #include "du5sem.h"

    /* semantic value */
    #define YYSTYPE mlc::MlaskalLval

	/* locations */
	#define YYLTYPE unsigned
	#define YYLLOC_DEFAULT(cur, rhs, n)	do { if(n) (cur)=(rhs)[1]; else (cur)=(rhs)[0]; } while(0)

    /* local stuff */
    using namespace mlc;
    
	/* error stuff */
    #define YYERROR_VERBOSE 1
    static void yyerror(YYLTYPE *line, mlc::MlaskalCtx *ctx, const char *m);
    
%}

%token DUTOK_PROGRAM			    /* program */
%token DUTOK_LABEL			    /* label */
%token DUTOK_CONST			    /* const */
%token DUTOK_TYPE			    /* type */
%token DUTOK_VAR			    /* var */
%token DUTOK_BEGIN			    /* begin */
%token DUTOK_END			    /* end */
%token DUTOK_PROCEDURE			    /* procedure */
%token DUTOK_FUNCTION			    /* function */
%token DUTOK_ARRAY			    /* array */
%token DUTOK_OF				    /* of */
%token DUTOK_GOTO			    /* goto */
%token DUTOK_IF				    /* if */
%token DUTOK_THEN			    /* then */
%token DUTOK_ELSE			    /* else */
%token DUTOK_WHILE			    /* while */
%token DUTOK_DO				    /* do */
%token DUTOK_REPEAT			    /* repeat */
%token DUTOK_UNTIL			    /* until */
%token DUTOK_FOR			    /* for */
%token DUTOK_OR				    /* or */
%token DUTOK_NOT			    /* not */
%token DUTOK_RECORD			    /* record */

/* literals */
%token<id_ci_>   DUTOK_IDENTIFIER			    /* identifier */
%token<int_ci_>  DUTOK_UINT		         	    /* unsigned integer */
%token<real_ci_> DUTOK_REAL			            /* real number */
%token<str_ci_>  DUTOK_STRING			        /* string */

/* delimiters */
%token DUTOK_SEMICOLON			    /* ; */
%token DUTOK_DOT			    /* . */
%token DUTOK_COMMA			    /* , */
%token DUTOK_EQ				    /* = */
%token DUTOK_COLON			    /* : */
%token DUTOK_LPAR			    /* ( */
%token DUTOK_RPAR			    /* ) */
%token DUTOK_DOTDOT			    /* .. */
%token DUTOK_LSBRA			    /* [ */
%token DUTOK_RSBRA			    /* ] */
%token DUTOK_ASSIGN			    /* := */

/* grouped operators and keywords */
%token<dtge_> DUTOK_OPER_REL			    /* <, <=, <>, >=, > */
%token<dtge_> DUTOK_OPER_SIGNADD		    /* +, - */
%token<dtge_> DUTOK_OPER_MUL			    /* *, /, div, mod, and */
%token<dtge_> DUTOK_FOR_DIRECTION		    /* to, downto */

%type<typeVal_>          type structuredTypeDef
%type<varQue_>           variables
%type<formalParamList_>  formalParametersList
%type<constVal_>         constant unsignedConstant
%type<fieldList_>        recordItemsList
%type<instructVal_>      variableLoad variableStor expression simpleExpression termsList term factorsList factor IDLessUnsignedConstant
%type<instructValQueue_> realParameters
%type<id_ci_>            functionHeader procedureHeader methodHeader
%type<code_>             program_block mainBlock commonBlock commandsList command goodCommand evilCommand goodCmd evilCmd assignCmd procedureCallCmd gotoCmd blockCmd goodConditionCmd goodWhileCmd repeatCmd goodForCmd nullCmd evilConditionCmd evilWhileCmd evilForCmd

%start mlaskal
%error-verbose
%pure-parser
%parse-param {mlc::MlaskalCtx *ctx}
%lex-param {mlc::MlaskalCtx *ctx}
%locations

%%



mlaskal:                DUTOK_PROGRAM DUTOK_IDENTIFIER DUTOK_SEMICOLON program_block DUTOK_DOT { ctx->tab->set_main_code($2, $4); }
		;

program_block:          labelsBlock constantsBlock typesBlock variablesBlock methodsBlock mainBlock { $$ = $6; }
		;
		
labelsBlock:            /* empty */
                       |DUTOK_LABEL labelsList DUTOK_SEMICOLON
        ;                          		
        
labelsList:             DUTOK_UINT                        { ctx->tab->add_label_entry(@1, $1, new_label(ctx)); }
                       |labelsList DUTOK_COMMA DUTOK_UINT { ctx->tab->add_label_entry(@3, $3, new_label(ctx)); }
        ;                               

constantsBlock:         /* empty */
                       |DUTOK_CONST constantsList
        ;                               
        
constantsList:          DUTOK_IDENTIFIER DUTOK_EQ constant DUTOK_SEMICOLON               { myAddConst(ctx, $1, $3, @1); }
                       |constantsList DUTOK_IDENTIFIER DUTOK_EQ constant DUTOK_SEMICOLON { myAddConst(ctx, $2, $4, @2); }
        ;        
        
typesBlock:             /* empty */
                       |DUTOK_TYPE typesList
        ;
        
typesList:              DUTOK_IDENTIFIER DUTOK_EQ type DUTOK_SEMICOLON           { if ($3.defined) ctx->tab->add_type(@1, $1, $3.typePointer); }
                       |typesList DUTOK_IDENTIFIER DUTOK_EQ type DUTOK_SEMICOLON { if ($4.defined) ctx->tab->add_type(@2, $2, $4.typePointer); }
        ;                               
        
variablesBlock:         /* empty */
                       |DUTOK_VAR variablesList
        ;
        
variablesList:          variables DUTOK_COLON type DUTOK_SEMICOLON               { myAddVar(ctx, &($1), $3, @1); }
                       |variablesList variables DUTOK_COLON type DUTOK_SEMICOLON { myAddVar(ctx, &($2), $4, @1); }
        ;  
        
variables:              DUTOK_IDENTIFIER                       { $$.push($1); }
                       |variables DUTOK_COMMA DUTOK_IDENTIFIER { $$ = $1; $$.push($3); }
        ;                                                            
        
methodsBlock:           /* empty */
                       |methodsList
        ;
        
methodsList:            methodHeader DUTOK_SEMICOLON commonBlock DUTOK_SEMICOLON             { ctx->tab->leave(@4); ctx->tab->set_subprogram_code($1, $3); }
                       |methodsList methodHeader DUTOK_SEMICOLON commonBlock DUTOK_SEMICOLON { ctx->tab->leave(@5); ctx->tab->set_subprogram_code($2, $4); }
        ;
        
methodHeader:           functionHeader  { $$ = $1; }
                       |procedureHeader { $$ = $1; }
        ;                       

mainBlock:              DUTOK_BEGIN commandsList DUTOK_END { $$ = $2; }
        ;
        
commandsList:           command { $$ = $1; }
                       |commandsList DUTOK_SEMICOLON command { $$ = $1; mlc::icblock_append_delete($$, $3); }
        ;
        
commonBlock:            labelsBlock constantsBlock typesBlock variablesBlock mainBlock { $$ = $5 }
        ;
        
procedureHeader:        DUTOK_PROCEDURE DUTOK_IDENTIFIER                                            { $$ = $2; ctx->tab->add_proc(@1, $2, create_parameter_list()); ctx->tab->enter(@2, $2); }
                       |DUTOK_PROCEDURE DUTOK_IDENTIFIER DUTOK_LPAR formalParametersList DUTOK_RPAR { $$ = $2; ctx->tab->add_proc(@1, $2, $4                     ); ctx->tab->enter(@2, $2); }
        ;
        
functionHeader:         DUTOK_FUNCTION DUTOK_IDENTIFIER DUTOK_COLON DUTOK_IDENTIFIER /* scalarTypeIdentifier */                                            { $$ = $2; myAddFunction(ctx, $2, create_parameter_list(), $4, @2); ctx->tab->enter(@2, $2); }
                       |DUTOK_FUNCTION DUTOK_IDENTIFIER DUTOK_LPAR formalParametersList DUTOK_RPAR DUTOK_COLON DUTOK_IDENTIFIER /* scalarTypeIdentifier */ { $$ = $2; myAddFunction(ctx, $2, $4                     , $7, @2); ctx->tab->enter(@2, $2); }
        ;
        
formalParametersList:   variables DUTOK_COLON DUTOK_IDENTIFIER /* typeIdentifier */           { $$ = myCreateParamList(ctx, &($1), $3, false, @3); }
                       |DUTOK_VAR variables DUTOK_COLON DUTOK_IDENTIFIER /* typeIdentifier */ { $$ = myCreateParamList(ctx, &($2), $4, true , @3); }
                       |formalParametersList DUTOK_SEMICOLON variables DUTOK_COLON DUTOK_IDENTIFIER /* typeIdentifier */           { $$ = $1; $$->append_and_kill(myCreateParamList(ctx, &($3), $5, false, @5)); }
                       |formalParametersList DUTOK_SEMICOLON DUTOK_VAR variables DUTOK_COLON DUTOK_IDENTIFIER /* typeIdentifier */ { $$ = $1; $$->append_and_kill(myCreateParamList(ctx, &($4), $6, true , @5)); }
        ;            
        
type:                   /* ordinalType */
                        DUTOK_IDENTIFIER /* typeIdentifier */ /* ordinalTypeIdentifier */ /* structuredTypeIdentifier */ 
                          {
                            mlc::symbol_pointer pType;
                            pType = ctx->tab->find_symbol($1);
                            if (!pType->access_type())
                            {
                              $$.defined     = false;
                            }
                            else
                            {
                              $$.typePointer = pType->access_type()->access_type()->type();
                              $$.defined     = true;
                            }
                          }
                       |structuredTypeDef
        ;
        
/* ordinalType:            DUTOK_IDENTIFIER POZNAMKAordinalTypeIdentifier 
        ; */
        
/* structuredType:         DUTOK_IDENTIFIER POZNAMKAstructuredTypeIdentifier
                       |DUTOK_RECORD recordItemsList DUTOK_SEMICOLON DUTOK_END
                       |DUTOK_RECORD recordItemsList DUTOK_END
                       |DUTOK_RECORD DUTOK_END
        ; */
        
structuredTypeDef:      DUTOK_RECORD recordItemsList DUTOK_SEMICOLON DUTOK_END { $$.typePointer = ctx->tab->create_record_type($2); $$.defined = true; }
                       |DUTOK_RECORD recordItemsList DUTOK_END                 { $$.typePointer = ctx->tab->create_record_type($2); $$.defined = true; }
                       |DUTOK_RECORD DUTOK_END                                 { $$.typePointer = ctx->tab->create_record_type(create_field_list()); $$.defined = true; }
        ;                       
        
recordItemsList:        variables DUTOK_COLON type                                 { $$ = myCreateFieldList(&($1), $3); }
                       |recordItemsList DUTOK_SEMICOLON variables DUTOK_COLON type { $$ = $1; $$->append_and_kill(myCreateFieldList(&($3), $5)); }
        ;
        
command:                goodCommand { $$ = $1 }
                       |evilCommand { $$ = $1 }
        ;                       

goodCommand:            labelForJump goodCmd { $$ = $2 }
                       |goodCmd              { $$ = $1 }
        ;
        
evilCommand:            labelForJump evilCmd { $$ = $2 }
                       |evilCmd              { $$ = $1 }
        ;                       
        
goodCmd:                assignCmd { $$ = $1; }
                       |procedureCallCmd { $$ = $1; }
                       |gotoCmd { $$ = $1; }
                       |blockCmd { $$ = $1; }
                       |goodConditionCmd { $$ = $1; }
                       |goodWhileCmd { $$ = $1; }
                       |repeatCmd { $$ = $1; }
                       |goodForCmd { $$ = $1; }
                       |nullCmd { $$ = $1; }
        ;
        
evilCmd:                evilConditionCmd { $$ = $1; }
                       |evilWhileCmd { $$ = $1; }
                       |evilForCmd { $$ = $1; }
        ;                               

/*                        assignCmd
                       |procedureCallCmd
                       |gotoCmd
                       |blockCmd
                       |conditionCmd
                       |whileCmd
                       |repeatCmd
                       |forCmd
                       |nullCmd
        ; */
        
labelForJump:           DUTOK_UINT DUTOK_COLON
        ;
        
assignCmd:              variableStor DUTOK_ASSIGN expression
                          { 
                            if ($1.hasAddress)
                            {
                              /* if ($1.typeCat != $3.typeCat)
                              {
                                if ((TCAT_REAL == $1.typeCat) && (TCAT_INT == $3.typeCat))
                                  $3.code->append_instruction(new ai::CVRTIR);
                                if ((TCAT_REAL == $3.typeCat) && (TCAT_INT == $1.typeCat))
                                  {
                                    $3.code->append_instruction(new ai::CVRTRI);
                                    error(DUERR_CONVERSION);
                                  }
                                if (!(((TCAT_REAL == $1.typeCat) && (TCAT_INT == $3.typeCat)) || 
                                      ((TCAT_REAL == $3.typeCat) && (TCAT_INT == $1.typeCat))))
                                  error(DUERR_CANNOTCONVERT);
                              } */
                              myTotalConversion($1.typeCat, $3.typeCat, &($3), @1);
                              $$ = $3.code;
                              icblock_append_delete($$, $1.code);
                            }
                            else 
                            {
                              $$ = $3.code;
                              error(DUERR_NOTLVALUE, @3);
                            }
                          }
                      /* |DUTOK_IDENTIFIER POZNAMKAfunctionIdentifier DUTOK_ASSIGN expression - variable muze byt i identifier */
        ;
        
procedureCallCmd:       DUTOK_IDENTIFIER /* procedureIdentifier */ { $$ = myMethodCall(ctx, $1, NULL, false, @1).code; }
                       |DUTOK_IDENTIFIER /* procedureIdentifier */ DUTOK_LPAR realParameters DUTOK_RPAR { $$ = myMethodCall(ctx, $1, &($3), false, @1).code; }
        ;
        
gotoCmd:                DUTOK_GOTO DUTOK_UINT { $$ = icblock_create(); }
        ;
        
blockCmd:               DUTOK_BEGIN commandsList DUTOK_END { $$ = icblock_create(); }
        ;        
        
goodConditionCmd:       DUTOK_IF booleanExpression DUTOK_THEN goodCommand DUTOK_ELSE goodCommand { $$ = icblock_create(); }
        ;
        
evilConditionCmd:       DUTOK_IF booleanExpression DUTOK_THEN command { $$ = icblock_create(); }
                       |DUTOK_IF booleanExpression DUTOK_THEN goodCommand DUTOK_ELSE evilCommand { $$ = icblock_create(); }
        ;
        
goodWhileCmd:           DUTOK_WHILE booleanExpression DUTOK_DO goodCommand { $$ = icblock_create(); }
        ;
        
evilWhileCmd:           DUTOK_WHILE booleanExpression DUTOK_DO evilCommand { $$ = icblock_create(); }
        ;        
        
repeatCmd:              DUTOK_REPEAT commandsList DUTOK_UNTIL booleanExpression { $$ = icblock_create(); }
        ;
        
goodForCmd:             DUTOK_FOR DUTOK_IDENTIFIER /* ordinalTypedVariableIdentifier */ DUTOK_ASSIGN ordinalTypedExpression DUTOK_FOR_DIRECTION ordinalTypedExpression DUTOK_DO goodCommand { $$ = icblock_create(); }
        ;
        
evilForCmd:             DUTOK_FOR DUTOK_IDENTIFIER /* ordinalTypedVariableIdentifier */ DUTOK_ASSIGN ordinalTypedExpression DUTOK_FOR_DIRECTION ordinalTypedExpression DUTOK_DO evilCommand { $$ = icblock_create(); }
        ;        
        
nullCmd:                /* empty */ { $$ = icblock_create(); }
        ;
        
variableStor:           DUTOK_IDENTIFIER { $$ = myIdentifierProcess(ctx, $1, true, @1); }
                       |variableStor DUTOK_DOT DUTOK_IDENTIFIER
        
variableLoad:           DUTOK_IDENTIFIER /* variableIdentifier , ale kvuli konfliktum v gramatice to muze byt i volani funkce */  { $$ = myIdentifierProcess(ctx, $1, false, @1); }
                       |variableLoad /* recordTypedVariable */ DUTOK_DOT DUTOK_IDENTIFIER /* recordItemIdentifier */
        ;
        
/* IDShortVariable:        * empty *
                       |IDShortVariable DUTOK_DOT DUTOK_IDENTIFIER */ /* recordItemIdentifier */ 
        ;                               
        
realParameters:         expression { $$.push($1); }
                      /* |variable - realParameters nemusi byt promenna nebo vyraz , ackoliv to ve specifikacie je , jenze vyraz je mimo jine i promenna */
                       |realParameters DUTOK_COMMA expression { $1.push($3); $$ = $1; }
                      /* |realParameters DUTOK_COMMA variable - opet totez , expression je mimo jine i variable */
        ;
        
expression:             simpleExpression DUTOK_OPER_REL simpleExpression { $$.code = myRelOperResult($2, &($1), &($3), @1); $$.typeCat = TCAT_BOOL; $$.hasAddress = false; }
                       |simpleExpression DUTOK_EQ simpleExpression { $$.code = myRelOperResult(-1, &($1), &($3), @1); $$.typeCat = TCAT_BOOL; $$.hasAddress = false; }
                       |simpleExpression { $$ = $1; }
        ;
        
simpleExpression:       DUTOK_OPER_SIGNADD termsList
						  {
						    $$ = $2;
						    $$.hasAddress = false;
						    if (DUTOKGE_MINUS == $1)
						    {
						      if (TCAT_INT  == $2.typeCat)
						        $2.code->append_instruction(new ai::MINUSI());
						      if (TCAT_REAL == $2.typeCat)
						        $2.code->append_instruction(new ai::MINUSR());
						    }
						    if (!((TCAT_INT == $2.typeCat) || (TCAT_REAL == $2.typeCat)))
						      error(DUERR_CANNOTCONVERT, @2);
						  }
                       |termsList { $$ = $1; }
        ;
        
termsList:              termsList DUTOK_OR term { $$ = myTermsListBuild(-1, &($1), &($3), @3); }
                       |termsList DUTOK_OPER_SIGNADD term { $$ = myTermsListBuild($2, &($1), &($3), @3); }
                       |term { $$ = $1; }
        ;
        
term:                   factorsList { $$ = $1; }
        ;
        
factorsList:            factorsList DUTOK_OPER_MUL factor { $$ = myFactorsListBuild($2, &($1), &($3), @3); }
                       |factor { $$ = $1; }
        ;
        
factor:                 IDLessUnsignedConstant /* konstanta muze byt jen identifikator , coz zahrnuji ostatni vetve */ { $$ = $1; }
                       |variableLoad /* IDShortVariable promenna zacina identifikatorem */ { $$ = $1; }
                       |DUTOK_IDENTIFIER /* functionIdentifier */ DUTOK_LPAR realParameters DUTOK_RPAR { $$ = myMethodCall(ctx, $1, &($3), true, @1); }
                      /* |DUTOK_IDENTIFIER POZNAMKAfunctionIdentifier */ /* variable pozere identifier */
                       |DUTOK_LPAR expression DUTOK_RPAR { $$ = $2; }
                       |DUTOK_NOT factor { (TCAT_BOOL == $2.typeCat) ? $2.code->append_instruction(new ai::NOT()) : error(DUERR_CANNOTCONVERT, @2); $$ = $2; $$.hasAddress = false; }
        ;
        
constant:               unsignedConstant              { $$ = $1; }
                       |DUTOK_OPER_SIGNADD DUTOK_UINT { $$.constType = _INT;  (DUTOKGE_MINUS == $1) ? $$.intVal  = ctx->tab->ls_int().add(-1 * (*$2))  : $$.intVal  = $2; }
                       |DUTOK_OPER_SIGNADD DUTOK_REAL { $$.constType = _REAL; (DUTOKGE_MINUS == $1) ? $$.realVal = ctx->tab->ls_real().add(-1 * (*$2)) : $$.realVal = $2; }
        ;
        
unsignedConstant:       DUTOK_IDENTIFIER /* constantIdentifier */ { $$.constType = _ID;   $$.id      = $1; }
                       |DUTOK_UINT                                { $$.constType = _INT;  $$.intVal  = $1; }
                       |DUTOK_REAL                                { $$.constType = _REAL; $$.realVal = $1; }
                       |DUTOK_STRING                              { $$.constType = _STR;  $$.strVal  = $1; }
        ;
        
IDLessUnsignedConstant: DUTOK_UINT   { $$.code = icblock_create(); $$.code->append_instruction(new ai::LDLITI($1)); $$.typeCat = TCAT_INT;  $$.hasAddress = false; }
                       |DUTOK_REAL   { $$.code = icblock_create(); $$.code->append_instruction(new ai::LDLITR($1)); $$.typeCat = TCAT_REAL; $$.hasAddress = false; }
                       |DUTOK_STRING { $$.code = icblock_create(); $$.code->append_instruction(new ai::LDLITS($1)); $$.typeCat = TCAT_STR;  $$.hasAddress = false; }
        ;                       
        
/* recordTypedVariable:             DUTOK_IDENTIFIER
        ;
        
variableIdentifier:              DUTOK_IDENTIFIER
        ;
        
ordinalTypedVariableIdentifier:  DUTOK_IDENTIFIER
        ;
        
scalarTypeIdentifier:            DUTOK_IDENTIFIER
        ;
        
ordinalTypeIdentifier:           DUTOK_IDENTIFIER
        ;
        
procedureIdentifier:             DUTOK_IDENTIFIER
        ;

constantIdentifier:              DUTOK_IDENTIFIER
        ;
        
recordItemIdentifier:            DUTOK_IDENTIFIER
        ;
        
functionIdentifier:              DUTOK_IDENTIFIER
        ;
        
structuredTypeIdentifier:        DUTOK_IDENTIFIER
        ;
        
typeIdentifier:                  DUTOK_IDENTIFIER
        ; */
        
booleanExpression:               expression
        ; 
        
ordinalTypedExpression:          expression
        ;             



%%

static void yyerror(unsigned *line, MlaskalCtx *, const char *m)
{
    error(DUERR_SYNTAX, *line, m);
}