%{
    #include "du6.h"
    #include "du6lval.hpp"
    #include "duerr.h"
    #include "du6tok.h"
    #include "du6sem.h"

    /* semantic value */
    #define YYSTYPE mlc::MlaskalLval

    /* locations */
    #define YYLTYPE unsigned
    #define YYLLOC_DEFAULT(cur, rhs, n)	do { if(n) (cur)=(rhs)[1]; else (cur)=(rhs)[0]; } while(0)

    /* local stuff */
    using namespace mlc;

    /* error stuff */
    #define YYERROR_VERBOSE 1
    static void yyerror(YYLTYPE *line, mlc::MlaskalCtx *ctx, const char *m);

    /* hack */
    #define YY_HACK
    #ifdef YY_HACK
		//#define short long
		#define YYINCLUDED_STDLIB_H
    #endif
%}

%token DUTOK_PROGRAM			/* program */
%token DUTOK_LABEL			    /* label */
%token DUTOK_CONST			    /* const */
%token DUTOK_TYPE			    /* type */
%token DUTOK_VAR			    /* var */
%token DUTOK_BEGIN			    /* begin */
%token DUTOK_END			    /* end */
%token DUTOK_PROCEDURE			/* procedure */
%token DUTOK_FUNCTION			/* function */
%token DUTOK_ARRAY			    /* array */
%token DUTOK_OF				    /* of */
%token DUTOK_GOTO			    /* goto */
%token DUTOK_IF				    /* if */
%token DUTOK_THEN			    /* then */
%token DUTOK_ELSE			    /* else */
%token DUTOK_WHILE			    /* while */
%token DUTOK_DO				    /* do */
%token DUTOK_REPEAT			    /* repeat */
%token DUTOK_UNTIL			    /* until */
%token DUTOK_FOR			    /* for */
%token DUTOK_OR				    /* or */
%token DUTOK_NOT			    /* not */

/* literals */
%token<id_ci_> DUTOK_IDENTIFIER			/* identifier */
%token<int_ci_> DUTOK_UINT			    /* unsigned integer */
%token<real_ci_> DUTOK_REAL			    /* real number */
%token<str_ci_> DUTOK_STRING			/* string */

/* delimiters */
%token DUTOK_SEMICOLON			/* ; */
%token DUTOK_DOT			    /* . */
%token DUTOK_COMMA			    /* , */
%token DUTOK_EQ				    /* = */
%token DUTOK_COLON			    /* : */
%token DUTOK_LPAR			    /* ( */
%token DUTOK_RPAR			    /* ) */
%token DUTOK_DOTDOT			    /* .. */
%token DUTOK_LSBRA			    /* [ */
%token DUTOK_RSBRA			    /* ] */
%token DUTOK_ASSIGN			    /* := */

/* grouped operators and keywords */
%token<dtge_> DUTOK_OPER_REL			    /* <, <=, <>, >=, > */
%token<dtge_> DUTOK_OPER_SIGNADD		    /* +, - */
%token<dtge_> DUTOK_OPER_MUL			    /* *, /, div, mod, and */
%token<dtge_> DUTOK_FOR_DIRECTION		    /* to, downto */

/* type */
%type<typ_>			typ typ_ordinal typ_struct		/* vraci hodnotu typu */
%type<konst_>		konstanta const_ordinal			/* vraci hodnotu konstanty */
%type<sign_>		sign							/* +1/-1 */
%type<list_typu_>	ordinals						/* seznam ordinalnich typu */
%type<list_prom_>	identifiers						/* seznam identifikatoru */
%type<list_param_>	formal_param1 formal_param		/* seznam formalnich parametru funkce */
/* type - kod */
%type<id_ci_>		zahlavi_proc zahlavi_fce		/* identifikator jmena procedury */
%type<instr2_> 		vyraz jedn_vyraz terms term faktor				/* ukazatel na instrukce a typ (vyraz) */
%type<instr2_>		lpromenna promenna
%type<asm_>			program_block blok prikazy prikaz prikaz_ prikaz_h prikaz_h_ prikaz_spol_ 		/* ukazatel na instrukce */
%type<param_instr_>	skut_param ordinaly				/* seznam skutench parametr */

%start mlaskal
%error-verbose
%pure-parser
%parse-param {mlc::MlaskalCtx *ctx}
%lex-param {mlc::MlaskalCtx *ctx}
%locations

%%

mlaskal:	  DUTOK_PROGRAM DUTOK_IDENTIFIER DUTOK_SEMICOLON program_block DUTOK_DOT
						{
							ctx->tab->set_main_code($2, $4);
						}
		;

program_block:	label_blok const_blok type_blok var_blok subrutins_blok DUTOK_BEGIN prikazy DUTOK_END
						{
							$$=$7;
						}
		;
blok:			label_blok const_blok type_blok var_blok DUTOK_BEGIN prikazy DUTOK_END
						{
							ctx->tab->leave(@7);
							$$=$6;
						}
		;

label_blok:		DUTOK_LABEL uint_comma DUTOK_SEMICOLON
		|
		;
uint_comma:		DUTOK_UINT
						{ ctx->tab->add_label_entry(@1,$1, new_label(ctx)); }
		|		uint_comma DUTOK_COMMA DUTOK_UINT
						{ ctx->tab->add_label_entry(@3,$3, new_label(ctx)); }
		;

const_blok:		DUTOK_CONST const_deklarace
		|
		;
const_deklarace: DUTOK_IDENTIFIER DUTOK_EQ konstanta DUTOK_SEMICOLON
						{ pridej_konst(ctx, @1, @3, $1, $3); }
		|		const_deklarace DUTOK_IDENTIFIER DUTOK_EQ konstanta DUTOK_SEMICOLON
						{ pridej_konst(ctx, @2, @4, $2, $4); }
		;

type_blok:		DUTOK_TYPE type_deklarace
		|
		;
type_deklarace: DUTOK_IDENTIFIER DUTOK_EQ typ DUTOK_SEMICOLON
						{ pridej_typ(ctx, @1, @3, $1, $3); }
		|		type_deklarace DUTOK_IDENTIFIER DUTOK_EQ typ DUTOK_SEMICOLON
						{ pridej_typ(ctx, @2, @4, $2, $4); }
		;

var_blok:	DUTOK_VAR var_deklarace
		|
		;
var_deklarace:	identifiers DUTOK_COLON typ DUTOK_SEMICOLON
						{ pridej_var(ctx, @1, @3, $1, $3); }
		|		var_deklarace identifiers DUTOK_COLON typ DUTOK_SEMICOLON
						{ pridej_var(ctx, @2, @4, $2, $4); }
		;
identifiers:	DUTOK_IDENTIFIER
						{ $$.push($1); }
		|		identifiers DUTOK_COMMA DUTOK_IDENTIFIER
						{ $1.push($3); $$=$1; }
		;

typ:			typ_ordinal
					{ $$=$1; }
		|		typ_struct
					{ $$=$1; }
		;
typ_ordinal:	const_ordinal DUTOK_DOTDOT const_ordinal
					{{
						mlc::ls_int_index lo, hi;
						bool _lo=true, _hi=true;
						symbol_pointer sp;

						switch($1.typ){
							case _INT:
								lo = $1.val_int;
								break;
							case _KONST:
								sp=ctx->tab->find_symbol($1.val_const);
								if(!!sp && !!sp->access_const()){
									lo = sp->access_const()->access_int_const()->int_value();
								}
								break;
							default:
								_lo=false;
								break;
						}
						switch($3.typ){
							case _INT:
								hi = $3.val_int;
								break;
							case _KONST:
								sp=ctx->tab->find_symbol($3.val_const);
								if(!!sp && !!sp->access_const()->access_int_const()){
									hi = sp->access_const()->access_int_const()->int_value();
								}
								break;
							default:
								_hi=false;
								break;
						}
						if(_lo && _hi){
							if(*lo>*hi){
								error(DUERR_BADRANGE, @1);
								swap(lo,hi);
							}
							$$.typ=_TYPE;
							$$.jmeno = $1.jmeno+".."+$3.jmeno;
							$$.val_type=ctx->tab->create_range_type(lo, hi);
						}else{
							$$.typ=_UNDEF_T;
							$$.val_type=ctx->tab->logical_undef();
						}
					}}
		|		DUTOK_IDENTIFIER
					{{	mlc::symbol_pointer sp=ctx->tab->find_symbol($1);
						if(!sp->access_type()){
							error(DUERR_NOTTYPE, @1, *$1);
							$$.typ=_UNDEF_T;
							$$.val_type=ctx->tab->logical_undef();
						}else{
							$$.typ=_TYPE;
							$$.val_type=sp->access_type()->access_type()->type();
						}
						$$.jmeno=*$1;
					}}
		;
typ_struct:		DUTOK_ARRAY DUTOK_LSBRA ordinals DUTOK_RSBRA DUTOK_OF typ
					{{
						mlc::type_pointer of_type=$6.val_type;
						bool			  ok=true;

						if($6.typ==_UNDEF_T){
							error(DUERR_NOTTYPE, @6, $6.jmeno.c_str());
							$$.typ=_UNDEF_T;
							$$.val_type=ctx->tab->logical_undef();
						}else{
							$$.typ=_TYPE;
							while(!$3.empty() && ok){
								if(  $3.top().typ!=_UNDEF_T &&
									!!$3.top().val_type->access_range()->access_range()
								){
									of_type=ctx->tab->create_array_type($3.top().val_type, of_type);
								}else{
									error(DUERR_NOTORDINAL, @3);
									ok=false;
									$$.typ=_UNDEF_T;
									$$.val_type=ctx->tab->logical_undef();
								}
								$3.pop();
							}
							if(ok) $$.val_type=of_type;
						}
						$$.jmeno="array [...] of " + $6.jmeno;
					}}
		;
const_ordinal:	DUTOK_OPER_SIGNADD DUTOK_IDENTIFIER
					{{
						symbol_pointer sp2=ctx->tab->find_symbol($2);
						if(!sp2 || !sp2->access_const()){
							error(DUERR_NOTCONST, @2, *$2);
							$$.typ=_UNDEF_K;
						}else{
							if(sp2->access_const()->type()->cat()!=TCAT_INT){
								error(DUERR_NOTINTEGRAL, @2, *$2);
								$$.typ=_UNDEF_K;
								$$.jmeno=*$2;
								$$.val_const = $2;
							}else{
								if($1==DUTOKGE_MINUS){
									$$.typ=_INT;
									$$.jmeno = "-"+*$2;
									$$.val_int = ctx->tab->ls_int().add(-*sp2->access_const()->access_int_const()->int_value());
								}else{
									$$.typ=_KONST;
									$$.jmeno= *$2;
									$$.val_const = $2;
								}
							}
						}
					}}
		|		DUTOK_IDENTIFIER
					{{
						symbol_pointer sp=ctx->tab->find_symbol($1);
						$$.jmeno=*$1;
						if(!sp || !sp->access_const()){
							error(DUERR_NOTCONST, @1, *$1);
							$$.typ=_UNDEF_K;
						}else{
							if(sp->access_const()->type()->cat()!=TCAT_INT){
								error(DUERR_NOTINTEGRAL, @1, *$1);
								$$.typ=_UNDEF_K;
								$$.val_const = $1;
							}else{
								$$.typ=_KONST;
								$$.val_const = $1;
							}
						}
					}}
		|		DUTOK_OPER_SIGNADD DUTOK_UINT
					{
						switch($1){
							case DUTOKGE_MINUS:
								$$.typ=_INT;
								$$.val_int=ctx->tab->ls_int().add(-*$2);
								break;
							case DUTOKGE_PLUS:
								$$.typ=_INT;
								$$.val_int=$2;
								break;
						}
					}
		|		DUTOK_UINT
					{ $$.typ=_INT; $$.val_int=$1; }
		;
ordinals:	typ_ordinal
					{ $$.push($1); }
		|	ordinals DUTOK_COMMA typ_ordinal
					{ $1.push($3); $$=$1; }
		;

subrutins_blok:	subrutins_blok zahlavi_proc DUTOK_SEMICOLON blok DUTOK_SEMICOLON
						{
							ctx->tab->set_subprogram_code($2, $4);
						}
		|		subrutins_blok zahlavi_fce  DUTOK_SEMICOLON blok DUTOK_SEMICOLON
						{
							ctx->tab->set_subprogram_code($2, $4);
						}
		|
		;
zahlavi_proc:	DUTOK_PROCEDURE DUTOK_IDENTIFIER DUTOK_LPAR formal_param DUTOK_RPAR
						{	pridej_proc(ctx, @1, $2, $4);
							ctx->tab->enter(@2, $2);
							$$=$2;
						}
		|		DUTOK_PROCEDURE DUTOK_IDENTIFIER
						{	pridej_proc(ctx, @1, $2, mlc::create_parameter_list());
							ctx->tab->enter(@2, $2);
							$$=$2;
						}
		;
zahlavi_fce:	DUTOK_FUNCTION DUTOK_IDENTIFIER DUTOK_LPAR formal_param DUTOK_RPAR DUTOK_COLON DUTOK_IDENTIFIER /* skalarni typ */
						{	pridej_func(ctx, @1, $2, $4, $7);
							ctx->tab->enter(@2, $2); 
							$$=$2
						}
		|		DUTOK_FUNCTION DUTOK_IDENTIFIER	DUTOK_COLON DUTOK_IDENTIFIER /* skalarni typ */
						{	pridej_func(ctx, @1, $2, mlc::create_parameter_list(), $4);
							ctx->tab->enter(@2, $2);
							$$=$2
						}
		;
formal_param:	formal_param1
						{ $$=$1; }
		|		formal_param DUTOK_SEMICOLON formal_param1
						{	$1->append_and_kill($3); $$=$1; }
		;
formal_param1:	DUTOK_VAR identifiers DUTOK_COLON DUTOK_IDENTIFIER
								{ $$=vytvor_list_param(ctx, @4, $2, $4, true); }
		|		          identifiers DUTOK_COLON DUTOK_IDENTIFIER
								{ $$=vytvor_list_param(ctx, @3, $1, $3, false); }
		;

prikazy:		prikaz
						{
							$$=$1;
						}
		|		prikazy DUTOK_SEMICOLON prikaz
						{ 
							$$=$1;
							mlc::icblock_append_delete($$,$3);
						}
		;
prikaz:		DUTOK_UINT DUTOK_COLON prikaz_
						{
							$$=$3;
							$$->add_label(*$1);
						}
		|	prikaz_
						{
							$$=$1;
						}
		;
prikaz_:	DUTOK_IF vyraz DUTOK_THEN prikaz
						{{
							if($2.tp_->cat()!=TCAT_BOOL){
								error(DUERR_CANNOTCONVERT,@2);
							}
							$$=$2.kod_;
							ic_label lb = new_label(ctx);
							$$->append_instruction_with_target(new ai::JF($$->end()), lb);
							$$->append_clear_block(*$4);
							$$->add_label(lb);
						}}
		|	DUTOK_IF vyraz DUTOK_THEN prikaz_h DUTOK_ELSE prikaz
						{{
							if($2.tp_->cat()!=TCAT_BOOL){
								error(DUERR_CANNOTCONVERT,@2);
							}
							$$ = $2.kod_;
							//then
							ic_label lb1 = new_label(ctx);
							$$->append_instruction_with_target(new ai::JF($$->end()), lb1);
							$$->append_clear_block(*$4);
							//else
							ic_label lb2 = new_label(ctx);
							$$->append_instruction_with_target(new ai::JMP($$->end()), lb2);
							$$->add_label(lb1);
							$$->append_clear_block(*$6);
							$$->add_label(lb2);
						}}
		|	DUTOK_WHILE vyraz DUTOK_DO prikaz
						{{
							if($2.tp_->cat()!=TCAT_BOOL){
								error(DUERR_CANNOTCONVERT, @2);
							}
							$$ = icblock_create();
							ic_label lb1 = new_label(ctx);
							$$->append_instruction_with_target(new ai::JMP($$->end()), lb1);
							
							ic_label lb2 = new_label(ctx);
							$$->add_label(lb2);
							$$->append_clear_block(*$4);
							
							$$->add_label(lb1);
							$$->append_clear_block(*$2.kod_);
							$$->append_instruction_with_target(new ai::JF($$->end()), lb2);
						}}
		|	DUTOK_FOR DUTOK_IDENTIFIER DUTOK_ASSIGN vyraz DUTOK_FOR_DIRECTION vyraz DUTOK_DO prikaz
						{ /* ASM!!! */	
							$$=$8;
						}
		|	prikaz_spol_
						{ 
							$$=$1;
						}
		;

prikaz_h:	DUTOK_UINT DUTOK_COLON prikaz_h_
						{ 
							$$=$3;
							$$->add_label(*$1);
						}
		|	prikaz_h_
						{ 
							$$=$1;
						}
		;
prikaz_h_:	DUTOK_IF vyraz DUTOK_THEN prikaz_h DUTOK_ELSE prikaz_h
						{{
							if($2.tp_->cat()!=TCAT_BOOL){
								error(DUERR_CANNOTCONVERT,@2);
							}
							$$ = $2.kod_;
							//then
							ic_label lb1 = new_label(ctx);
							$$->append_instruction_with_target(new ai::JF($$->end()), lb1);
							$$->append_clear_block(*$4);
							//else
							ic_label lb2 = new_label(ctx);
							$$->append_instruction_with_target(new ai::JMP($$->end()), lb2);
							$$->add_label(lb1);
							$$->append_clear_block(*$6);
							$$->add_label(lb2);
						}}
		|	DUTOK_WHILE vyraz DUTOK_DO prikaz_h
						{{
							if($2.tp_->cat()!=TCAT_BOOL){
								error(DUERR_CANNOTCONVERT, @2);
							}
							$$ = icblock_create();
							ic_label lb1 = new_label(ctx);
							$$->append_instruction_with_target(new ai::JMP($$->end()), lb1);
							
							ic_label lb2 = new_label(ctx);
							$$->add_label(lb2);
							$$->append_clear_block(*$4);
							
							$$->add_label(lb1);
							$$->append_clear_block(*$2.kod_);
							$$->append_instruction_with_target(new ai::JF($$->end()), lb2);
						}}
		|	DUTOK_FOR DUTOK_IDENTIFIER DUTOK_ASSIGN vyraz DUTOK_FOR_DIRECTION vyraz DUTOK_DO prikaz_h
						{ /* ASM!!! */	
							$$=$8;
						}
		|	prikaz_spol_
						{	
							$$=$1;
						}
		;

prikaz_spol_:	lpromenna DUTOK_ASSIGN vyraz
						{
							if(!$1.lval_){
								error(DUERR_NOTLVALUE, @1);
							}else{
								//pokud nesedi typy:
								if($1.typ_ != $3.typ_){
									//pokud jde konvertovat:
									if($1.typ_==TCAT_REAL && convertable2real($3.typ_))
										$3.kod_->append_instruction(new ai::CVRTIR());
									else
									if($1.typ_==TCAT_INT && convertable2real($3.typ_)){ //tj. $3 je int/range
										$3.kod_->append_instruction(new ai::CVRTRI());
										error(DUERR_CONVERSION, @1);
									}else{
										error(DUERR_CANNOTCONVERT, @1);
									}
								}
								$$=$3.kod_;
								icblock_append_delete($$, $1.kod_);
							}
						}
		|	DUTOK_IDENTIFIER /*procedury*/
						{{
							ParamInstr pi_pom;	
							$$=sub_call(ctx, @1, @1, $1, pi_pom, true).kod_;
						}}
		|	DUTOK_IDENTIFIER /*procedury*/ DUTOK_LPAR skut_param DUTOK_RPAR
						{ 
							$$=sub_call(ctx, @1, @3, $1, $3, true).kod_;
						}
		|	DUTOK_GOTO DUTOK_UINT
						{	 /* ASM!!! */	
							$$=mlc::icblock_create();
							//if(!ctx->tab->find_label($2)){}
							//$$->append_instruction(new ai::JMP($2));
						}
		|	DUTOK_BEGIN prikazy DUTOK_END
						{	
							$$=$2;
						}
		|	DUTOK_REPEAT prikazy DUTOK_UNTIL vyraz
						{{
							if($4.tp_->cat()!=TCAT_BOOL){
								error(DUERR_CANNOTCONVERT,@4);
							}
							$$ = icblock_create();
							ic_label lb = new_label(ctx);
							$$->add_label(lb);
							$$->append_clear_block(*$2);
							$$->append_clear_block(*$4.kod_);
							$$->append_instruction_with_target(new ai::JF($$->end()), lb);
						}}
		|
						{
							$$=mlc::icblock_create();
						}
		;

lpromenna:	DUTOK_IDENTIFIER
						{{
							$$.lval_=true;											
							$$.kod_=mlc::icblock_create();
							$$.typ_=mlc::TCAT_UNDEF;
							$$.tp_=type_pointer();
							symbol_pointer sp=ctx->tab->find_symbol($1);
							if(!sp){
								error(DUERR_NOTVAR, @1, *$1);
							}else{
								switch(sp->kind()){
									case SKIND_CONST:
									case SKIND_TYPE:
									case SKIND_UNDEF:
									case SKIND_PROCEDURE:
										break;
									case SKIND_FUNCTION:
										{
										if(ctx->tab->my_function_name()==$1){
											$$.tp_=ctx->tab->find_symbol($1)->access_function()->type();
											switch($$.typ_=$$.tp_->cat()){
												case TCAT_BOOL:
													$$.kod_->append_instruction(new ai::LSTB(ctx->tab->my_return_address()));
													break;
												case TCAT_INT:
												case TCAT_RANGE:
													$$.kod_->append_instruction(new ai::LSTI(ctx->tab->my_return_address()));
													break;
												case TCAT_REAL:
													$$.kod_->append_instruction(new ai::LSTR(ctx->tab->my_return_address()));
													break;
												case TCAT_STR:
													$$.kod_->append_instruction(new ai::LSTS(ctx->tab->my_return_address()));
													break;
												case TCAT_ARRAY:
													/* ASM!!! */
													break;
												case TCAT_UNDEF:
												default:
													;
											}
										}else{
											error(DUERR_NOTLVALUE, @1);
										}
										break;
										}
									case SKIND_GLOBAL_VARIABLE:
										$$.tp_=sp->access_variable()->type();
										switch($$.typ_=$$.tp_->cat()){
											case mlc::TCAT_ARRAY:
												$$.kod_->append_instruction(new ai::GREF(sp->access_variable()->address()));
												break;
											case mlc::TCAT_BOOL:
												$$.kod_->append_instruction(new ai::GSTB(sp->access_variable()->address()));
												break;
											case mlc::TCAT_INT:
											case mlc::TCAT_RANGE:
												$$.kod_->append_instruction(new ai::GSTI(sp->access_variable()->address()));
												break;
											case mlc::TCAT_REAL:
												$$.kod_->append_instruction(new ai::GSTR(sp->access_variable()->address()));
												break;
											case mlc::TCAT_STR:
												$$.kod_->append_instruction(new ai::GSTS(sp->access_variable()->address()));
												break;
											case mlc::TCAT_UNDEF:
												break;
										}
										break;
									case SKIND_LOCAL_VARIABLE:
										$$.tp_=sp->access_variable()->type();
										switch($$.typ_=$$.tp_->cat()){
											case mlc::TCAT_ARRAY:
												$$.kod_->append_instruction(new ai::LREF(sp->access_variable()->address()));
												break;
											case mlc::TCAT_BOOL:
												$$.kod_->append_instruction(new ai::LSTB(sp->access_variable()->address()));
												break;
											case mlc::TCAT_INT:
											case mlc::TCAT_RANGE:
												$$.kod_->append_instruction(new ai::LSTI(sp->access_variable()->address()));
												break;
											case mlc::TCAT_REAL:
												$$.kod_->append_instruction(new ai::LSTR(sp->access_variable()->address()));
												break;
											case mlc::TCAT_STR:
												$$.kod_->append_instruction(new ai::LSTS(sp->access_variable()->address()));
												break;
											case mlc::TCAT_UNDEF:
												break;
										}
										break;
									case SKIND_PARAMETER_BY_REFERENCE:
										$$.tp_=sp->access_parameter_by_reference()->type();
										$$.typ_=$$.tp_->cat();
										/* ASM!!! */
										break;
								}
							}
						}}
		|	lpromenna DUTOK_LSBRA ordinaly DUTOK_RSBRA /* byref */
						{
							//$$=$1;
							$$=access_pole(ctx, @1, $1, $3);
							switch($$.tp_->cat()){
								case TCAT_INT:
								case TCAT_RANGE:
									$$.kod_->append_instruction(new ai::XSTI());
									break;
								case TCAT_BOOL:
									$$.kod_->append_instruction(new ai::XSTB());
									break;
								case TCAT_REAL:
									$$.kod_->append_instruction(new ai::XSTR());
									break;
								case TCAT_STR:
									$$.kod_->append_instruction(new ai::XSTS());
									break;
								default:
									break;
							}
						}
		;
			
promenna:	DUTOK_IDENTIFIER
						{{
							$$.kod_=mlc::icblock_create();
							$$.typ_=mlc::TCAT_UNDEF;
							$$.tp_=type_pointer();
							symbol_pointer sp=ctx->tab->find_symbol($1);
							if(!sp){
								error(DUERR_NOTVAR, @1, *$1);
							}else{
								switch(sp->kind()){
									case SKIND_TYPE:
									case SKIND_UNDEF:
									case SKIND_PROCEDURE:
										error(DUERR_NOTVAR, @1, *$1);
										break;
									case SKIND_FUNCTION:
										{
										ParamInstr pi_pom;
										delete $$.kod_;
										$$=sub_call(ctx, @1, @1, $1, pi_pom, false);
										break;
										}
									case SKIND_GLOBAL_VARIABLE:
										$$.tp_=sp->access_variable()->type();
										switch($$.typ_=$$.tp_->cat()){
											case mlc::TCAT_ARRAY:
												$$.kod_->append_instruction(new ai::GREF(sp->access_variable()->address()));												
												break;
											case mlc::TCAT_BOOL:
												$$.kod_->append_instruction(new ai::GLDB(sp->access_variable()->address()));
												break;
											case mlc::TCAT_INT:
											case mlc::TCAT_RANGE:
												$$.kod_->append_instruction(new ai::GLDI(sp->access_variable()->address()));
												break;
											case mlc::TCAT_REAL:
												$$.kod_->append_instruction(new ai::GLDR(sp->access_variable()->address()));
												break;
											case mlc::TCAT_STR:
												$$.kod_->append_instruction(new ai::GLDS(sp->access_variable()->address()));
												break;
											case mlc::TCAT_UNDEF:
												break;
										}
										break;
									case SKIND_LOCAL_VARIABLE:
										$$.tp_=sp->access_variable()->type();
										switch($$.typ_=$$.tp_->cat()){
											case mlc::TCAT_ARRAY:
												$$.kod_->append_instruction(new ai::LREF(sp->access_variable()->address()));
												break;
											case mlc::TCAT_BOOL:
												$$.kod_->append_instruction(new ai::LLDB(sp->access_variable()->address()));
												break;
											case mlc::TCAT_INT:
											case mlc::TCAT_RANGE:
												$$.kod_->append_instruction(new ai::LLDI(sp->access_variable()->address()));
												break;
											case mlc::TCAT_REAL:
												$$.kod_->append_instruction(new ai::LLDR(sp->access_variable()->address()));
												break;
											case mlc::TCAT_STR:
												$$.kod_->append_instruction(new ai::LLDS(sp->access_variable()->address()));
												break;
											case mlc::TCAT_UNDEF:
												break;
										}
										break;
									case SKIND_PARAMETER_BY_REFERENCE:
										$$.tp_=sp->access_parameter_by_reference()->type();
										$$.typ_=$$.tp_->cat();
										/* ASM!!! */
										break;
									case SKIND_CONST:
										$$.tp_=sp->access_const()->type();
										switch($$.typ_=$$.tp_->cat()){
											case mlc::TCAT_ARRAY:
												//??? - asi neni !!!
												break;
											case mlc::TCAT_BOOL:
												$$.kod_->append_instruction(new ai::LDLITB(sp->access_const()->access_bool_const()->bool_value()?1:0));
												break;
											case mlc::TCAT_RANGE:
											case mlc::TCAT_INT:
												$$.kod_->append_instruction(new ai::LDLITI(sp->access_const()->access_int_const()->int_value()));
												break;
											case mlc::TCAT_REAL:
												$$.kod_->append_instruction(new ai::LDLITR(sp->access_const()->access_real_const()->real_value()));
												break;
											case mlc::TCAT_STR:
												$$.kod_->append_instruction(new ai::LDLITS(sp->access_const()->access_str_const()->str_value()));
												break;
											case mlc::TCAT_UNDEF:
												break;
										}
										break;
								}
							}
						}}
		|	promenna DUTOK_LSBRA ordinaly DUTOK_RSBRA /* byval */
						{
							$$=access_pole(ctx, @1, $1, $3);
							switch($$.tp_->cat()){
								case TCAT_INT:
								case TCAT_RANGE:
									$$.kod_->append_instruction(new ai::XLDI());
									break;
								case TCAT_BOOL:
									$$.kod_->append_instruction(new ai::XLDB());
									break;
								case TCAT_REAL:
									$$.kod_->append_instruction(new ai::XLDR());
									break;
								case TCAT_STR:
									$$.kod_->append_instruction(new ai::XLDS());
									break;
								default:
									break;
							}
							
						}
ordinaly:	vyraz
						{	$$.push($1);
							switch($1.typ_){
								case TCAT_INT:
								case TCAT_RANGE:
									break;
								default:
									error(DUERR_NOTORDINAL, @1);
									break;
							}
						}
		|	ordinaly DUTOK_COMMA vyraz
						{	$$=$1;
							$$.push($3);
							switch($3.typ_){
								case TCAT_INT:
								case TCAT_RANGE:
									break;
								default:
									error(DUERR_NOTORDINAL, @3);
							}
						}
		;
skut_param:	vyraz
						{
							$$.push($1);
						}
		|	skut_param DUTOK_COMMA vyraz
						{
							$$=$1;
							$$.push($3);
						}
		;
vyraz:		jedn_vyraz DUTOK_OPER_REL jedn_vyraz
						{
							$$.typ_=TCAT_BOOL;
							$$.kod_=relace(@1, @3, $1, $3, $2);
							$$.lval_=false;
						}
		|	jedn_vyraz DUTOK_EQ jedn_vyraz
						{
							$$.typ_=TCAT_BOOL;
							$$.kod_=relace(@1, @3, $1, $3, -1);
							$$.lval_=false;
						}
		|	jedn_vyraz
						{
							$$=$1;
						}
		;
jedn_vyraz:	sign terms
						{
							$$=$2;
							if($1==-1){
								switch($$.typ_){
								case TCAT_INT:
									if($1==-1){
										$$.lval_=false;
										$$.kod_->append_instruction(new ai::MINUSI());
									}
									break;
								case TCAT_REAL:
									if($1==-1){
										$$.lval_=false;
										$$.kod_->append_instruction(new ai::MINUSR());
									}
									break;
								}
							}
							if($1!=0){
								switch($$.typ_){
								case TCAT_RANGE:
								case TCAT_BOOL:
								case TCAT_ARRAY:
								case TCAT_UNDEF:
									error(DUERR_CANNOTCONVERT, @2);
									break;
								}
							}
						}
		;
terms:		term
						{	
							$$=$1;
						}
		|	terms DUTOK_OPER_SIGNADD term
						{
							$$=secti(@1, @3, $1, $2, $3);
						}
		|	terms DUTOK_OR term
						{
							$$=secti(@1,@3, $1,-1/*OR*/,$3);
						}
		;
term:		faktor
						{ 
							$$=$1;
						}
		|	term DUTOK_OPER_MUL faktor
						{ /* "*,/,mod,div,and" */
							$$=$1;
							$$.lval_=false;
							if($1.typ_==TCAT_STR)
								error(DUERR_CANNOTCONVERT,@1);
							else if($3.typ_==TCAT_STR)
								error(DUERR_CANNOTCONVERT,@3);
							else{
								switch($2){
									case DUTOKGE_AND:			/* and */
										$$.typ_=TCAT_BOOL;
										if($1.typ_==TCAT_BOOL && $3.typ_==TCAT_BOOL){
											mlc::icblock_append_delete($$.kod_,$3.kod_);
											$$.kod_->append_instruction(new ai::AND());
										}else{
											//error(nelze nebool);
										}
										break;
									case DUTOKGE_MOD:			/* mod */
									case DUTOKGE_DIV:			/* div */
										$$.typ_=TCAT_INT;
										if($1.typ_==TCAT_INT && $3.typ_==TCAT_INT){
											mlc::icblock_append_delete($$.kod_,$3.kod_);
											if($2==DUTOKGE_MOD)
												$$.kod_->append_instruction(new ai::MODI());
											else
												$$.kod_->append_instruction(new ai::DIVI());
										}
										break;
									case DUTOKGE_SOLIDUS:		/* / */
										$$.typ_=TCAT_REAL;
										if($1.typ_!=TCAT_INT && $1.typ_!=TCAT_REAL){
											error(DUERR_CANNOTCONVERT,@1);
											break;
										}
										if($3.typ_!=TCAT_INT && $3.typ_!=TCAT_REAL){
											error(DUERR_CANNOTCONVERT,@3);
											break;
										}
										if($1.typ_==TCAT_INT)
											$$.kod_->append_instruction(new ai::CVRTIR());
										if($3.typ_==TCAT_INT)
											$3.kod_->append_instruction(new ai::CVRTIR());
										mlc::icblock_append_delete($$.kod_,$3.kod_);
										$$.kod_->append_instruction(new ai::DIVR());
										break;
									case DUTOKGE_ASTERISK:		/* * */
										if($1.typ_!=TCAT_INT && $1.typ_!=TCAT_REAL){
											error(DUERR_CANNOTCONVERT,@1);
											break;
										}
										if($3.typ_!=TCAT_INT && $3.typ_!=TCAT_REAL){
											error(DUERR_CANNOTCONVERT,@3);
											break;
										}
										switch($1.typ_){
											case TCAT_INT:
												switch($3.typ_){
													case TCAT_INT:
														$$.typ_=TCAT_INT;
														mlc::icblock_append_delete($$.kod_,$3.kod_);
														$$.kod_->append_instruction(new ai::MULI());
														break;
													case TCAT_REAL:
														$$.typ_=TCAT_REAL;
														$$.kod_->append_instruction(new ai::CVRTIR());
														mlc::icblock_append_delete($$.kod_,$3.kod_);
														$$.kod_->append_instruction(new ai::MULR());
														break;
												}
												break;
											case TCAT_REAL:
												switch($3.typ_){
													case TCAT_INT:
														$$.typ_=TCAT_REAL;
														mlc::icblock_append_delete($$.kod_,$3.kod_);
														$$.kod_->append_instruction(new ai::CVRTIR());
														$$.kod_->append_instruction(new ai::MULR());
														break;
													case TCAT_REAL:
														$$.typ_=TCAT_REAL;
														mlc::icblock_append_delete($$.kod_,$3.kod_);
														$$.kod_->append_instruction(new ai::MULR());
														break;
												}
												break;
										}
										break;
									default:
										//error(neznamy operator);
										break;
								}
							}
						}
		;
faktor:		DUTOK_STRING
						{
							$$.typ_=TCAT_STR;
							$$.kod_=icblock_create();
							$$.kod_->append_instruction(new ai::LDLITS($1));
							$$.lval_=false;
						}
		|	DUTOK_UINT
						{
							$$.typ_=TCAT_INT;
							$$.kod_=icblock_create();
							$$.kod_->append_instruction(new ai::LDLITI($1));
							$$.lval_=false;
						}
		|	DUTOK_REAL
						{
							$$.typ_=TCAT_REAL;
							$$.kod_=icblock_create();
							$$.kod_->append_instruction(new ai::LDLITR($1));
							$$.lval_=false;
						}
		|	promenna
						{ 
							$$=$1;
						}
		|	DUTOK_IDENTIFIER DUTOK_LPAR skut_param DUTOK_RPAR
						{
							$$=sub_call(ctx, @1, @3, $1, $3, false);
							$$.lval_=false;
						}
		|	DUTOK_LPAR vyraz DUTOK_RPAR
						{ 
							$$=$2;
							$$.lval_=false;
						}
		|	DUTOK_NOT faktor
						{ 
							$$=$2;
							$$.lval_=false;
							if($$.typ_==TCAT_BOOL)
								$$.kod_->append_instruction(new ai::NOT());
							else
								error(DUERR_CANNOTCONVERT, @2);
						}
		;

konstanta:		DUTOK_STRING		
						{ $$.typ=_STRING; $$.val_string=$1; }
		|		sign DUTOK_UINT		
						{	$$.typ=_INT;	  
							if($1==-1)
								$$.val_int=ctx->tab->ls_int().add($1* (*$2)); 
							else
								$$.val_int=$2;
						}
		|		sign DUTOK_REAL		
						{	$$.typ=_REAL;
							if($1==-1)
								$$.val_real=ctx->tab->ls_real().add($1* (*$2)); 
							else
								$$.val_real=$2;
						}
		|		DUTOK_IDENTIFIER	
						{ $$.typ=_KONST;  $$.val_const =$1; }
		;
sign:			DUTOK_OPER_SIGNADD
					{
						switch($1){
							case DUTOKGE_MINUS:
								$$ = -1;
								break;
							case DUTOKGE_PLUS:
								$$ =  1; // +1
								break;
						}
					}
		|			{ $$=0; }
		;
%%

static void yyerror(unsigned *line, MlaskalCtx *, const char *m)
{
    error(DUERR_SYNTAX, *line, m);
}
